"""Meeting follow-up SLA breach scout (Salesforce-first)."""

from __future__ import annotations

import time
from datetime import date, datetime, timedelta, timezone
from typing import Any, Dict, List, Tuple

from .base import BaseScoutCheck, PreflightResult
from ..crm_provider import Filter, FilterExpr, FilterOp, OrGroup
from ..models import EvidenceRow, RecommendedAction, ScoutFinding
from ..utils import chunked


class MeetingFollowupBreachCheck(BaseScoutCheck):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self._recommended_actions: list[RecommendedAction] = []
        self._action_keys: set[str] = set()

    def preflight(self) -> PreflightResult:
        base = super().preflight()
        if base.unsupported:
            return base

        if not self.provider.supports_object("Event"):
            return PreflightResult(ok=False, unsupported=True, reason="Unsupported object: Event")

        available = self._available_fields("Event")
        if "id" not in available:
            return PreflightResult(ok=False, unsupported=True, reason="Event.Id field unavailable")
        if "enddatetime" not in available and "activitydate" not in available:
            return PreflightResult(
                ok=False,
                unsupported=True,
                reason="Event.EndDateTime or Event.ActivityDate required for follow-up SLA checks",
            )
        return PreflightResult(ok=True)

    def build_count_filters(self) -> Tuple[str, List[FilterExpr]]:
        return "Event", self._event_filters(self._available_fields("Event"))

    def build_sample_request(self) -> Tuple[str, List[str], List[FilterExpr], str | None, str, bool]:
        available = self._available_fields("Event")
        fields, order_by = self._event_fields(available)
        return "Event", fields, self._event_filters(available), order_by, "DESC", False

    def evaluate(self, rows: List[Dict[str, Any]]) -> List[EvidenceRow]:
        del rows
        return []

    def execute(self) -> ScoutFinding:  # type: ignore[override]
        start = time.perf_counter()
        self._recommended_actions = []
        self._action_keys = set()

        try:
            preflight = self.preflight()
            if preflight.unsupported:
                return self._unsupported_finding(preflight.reason or "Unsupported")

            available = self._available_fields("Event")
            filters = self._event_filters(available)
            total_scanned = self.provider.count("Event", filters)
            self._collect_provider_meta()

            evidence_all: list[EvidenceRow] = []
            if total_scanned > 0:
                query_limit = self._query_limit()
                if total_scanned > query_limit:
                    self._warnings.append(
                        f"Event scan capped at {query_limit} rows out of {total_scanned}; findings may be partial."
                    )

                fields, order_by = self._event_fields(available)
                rows = self.provider.query(
                    "Event",
                    fields,
                    filters,
                    order_by=order_by,
                    order_dir="DESC",
                    nulls_first=False,
                    limit=query_limit,
                )
                self._collect_provider_meta()

                followup_index = self._load_followup_touch_index(rows)
                now = datetime.now(timezone.utc)
                for row in rows:
                    candidate = self._evaluate_event_row(row, followup_index, now)
                    if candidate is not None:
                        evidence_all.append(candidate)

            findings_count = len(evidence_all)
            evidence = evidence_all[: self.top_n]
            status = "findings" if findings_count > 0 else "ok"
            duration_ms = int((time.perf_counter() - start) * 1000)
            return ScoutFinding(
                scout_name=self.definition.name,
                display_name=self.definition.display_name,
                description=self.definition.description,
                category=self.definition.category,
                severity=self.definition.severity_default,
                total_records_scanned=total_scanned,
                findings_count=findings_count,
                evidence=evidence,
                recommended_actions=self.get_recommended_actions(),
                soql_queries=self._soql_queries,
                duration_ms=duration_ms,
                parameters_used=self.params,
                status=status,
                warnings=self._warnings,
            )
        except Exception as exc:
            duration_ms = int((time.perf_counter() - start) * 1000)
            return self._error_finding(str(exc), duration_ms=duration_ms)

    def get_recommended_actions(self) -> List[RecommendedAction]:
        return list(self._recommended_actions)

    def _evaluate_event_row(
        self,
        row: dict[str, Any],
        followup_index: dict[str, list[datetime]],
        now: datetime,
    ) -> EvidenceRow | None:
        event_id = str(row.get("Id") or "").strip()
        if not event_id:
            return None

        event_end = self._event_end_time(row)
        if event_end is None:
            return None

        sla_hours = self._sla_hours_for_event(row)
        if sla_hours <= 0:
            return None
        followup_deadline = event_end + timedelta(hours=sla_hours)
        if now <= followup_deadline:
            return None

        related_ids = self._event_related_ids(row)
        touches = self._related_touches(followup_index, related_ids)
        if any(event_end <= touch <= followup_deadline for touch in touches):
            return None

        owner_id = str(row.get("OwnerId") or "").strip()
        reason = (
            f"Meeting ended at {event_end.isoformat()} with no logged follow-up activity within "
            f"{sla_hours} hours."
        )
        self._queue_action(event_id=event_id, owner_id=owner_id, sla_hours=sla_hours, followup_deadline=followup_deadline)

        return EvidenceRow(
            record_id=event_id,
            sobject="Event",
            fields={
                "Subject": row.get("Subject"),
                "OwnerId": owner_id,
                "WhoId": row.get("WhoId"),
                "WhatId": row.get("WhatId"),
                "EndDateTime": row.get("EndDateTime") or row.get("ActivityDate"),
                "followup_sla_hours": sla_hours,
                "followup_deadline": followup_deadline.isoformat(),
            },
            reason=reason,
            reason_code="MEETING_FOLLOWUP_BREACH",
            severity=self.definition.severity_default,
            related_ids=related_ids,
        )

    def _queue_action(
        self,
        *,
        event_id: str,
        owner_id: str,
        sla_hours: int,
        followup_deadline: datetime,
    ) -> None:
        if event_id in self._action_keys:
            return
        self._action_keys.add(event_id)
        self._recommended_actions.append(
            RecommendedAction(
                action_type="followup_enforcement",
                label="Enforce meeting follow-up",
                risk_level="low",
                details={
                    "reason_code": "MEETING_FOLLOWUP_BREACH",
                    "event_id": event_id,
                    "owner_id": owner_id,
                    "due_at": datetime.now(timezone.utc).isoformat(),
                    "followup_sla_hours": sla_hours,
                    "followup_deadline": followup_deadline.isoformat(),
                    "apply_stage_hold": False,
                    "notify_slack": False,
                },
            )
        )

    def _load_followup_touch_index(self, events: list[dict[str, Any]]) -> dict[str, list[datetime]]:
        index: dict[str, list[datetime]] = {}
        if not events:
            return index

        who_ids: set[str] = set()
        what_ids: set[str] = set()
        for row in events:
            who_id = str(row.get("WhoId") or "").strip()
            what_id = str(row.get("WhatId") or "").strip()
            if who_id:
                who_ids.add(who_id)
            if what_id:
                what_ids.add(what_id)

        lookback_days = max(1, self._to_int(self.params.get("lookback_days"), default=14))
        self._record_task_touches(index, who_ids, what_ids, lookback_days)
        if bool(self.params.get("include_email_messages", True)):
            self._record_email_touches(index, who_ids | what_ids, lookback_days)
        if bool(self.params.get("include_notes", True)):
            self._record_note_touches(index, who_ids | what_ids, lookback_days)
        if bool(self.params.get("include_opportunity_updates", True)):
            self._record_opportunity_touches(index, what_ids, lookback_days)
        return index

    def _record_task_touches(
        self,
        index: dict[str, list[datetime]],
        who_ids: set[str],
        what_ids: set[str],
        lookback_days: int,
    ) -> None:
        if not self.provider.supports_object("Task"):
            return
        available = self._available_fields("Task")
        task_id_sets: list[tuple[str, set[str]]] = []
        if "whoid" in available and who_ids:
            task_id_sets.append(("WhoId", who_ids))
        if "whatid" in available and what_ids:
            task_id_sets.append(("WhatId", what_ids))
        if not task_id_sets:
            return

        timestamp_field = "CreatedDate" if "createddate" in available else ("LastModifiedDate" if "lastmodifieddate" in available else "")
        if not timestamp_field:
            self._warnings.append("Task.CreatedDate/LastModifiedDate unavailable; task follow-up evidence skipped.")
            return

        for field_name, id_set in task_id_sets:
            for id_chunk in chunked(sorted(id_set), size=200):
                filters: list[FilterExpr] = [
                    Filter(field_name, FilterOp.IN, list(id_chunk)),
                    Filter(timestamp_field, FilterOp.LAST_N_DAYS, lookback_days),
                ]
                fields = [field_name, timestamp_field]
                if field_name == "WhoId" and "whatid" in available:
                    fields.append("WhatId")
                if field_name == "WhatId" and "whoid" in available:
                    fields.append("WhoId")

                rows = self.provider.query(
                    "Task",
                    fields,
                    filters,
                    order_by=timestamp_field,
                    order_dir="DESC",
                    limit=max(self.top_n * 20, 300),
                )
                self._collect_provider_meta()
                for row in rows:
                    touched_at = self._parse_datetime(row.get(timestamp_field))
                    if touched_at is None:
                        continue
                    for rel_field in ("WhoId", "WhatId"):
                        rel_id = str(row.get(rel_field) or "").strip()
                        if rel_id:
                            self._add_touch(index, rel_id, touched_at)

    def _record_email_touches(
        self,
        index: dict[str, list[datetime]],
        related_ids: set[str],
        lookback_days: int,
    ) -> None:
        if not related_ids or not self.provider.supports_object("EmailMessage"):
            return
        available = self._available_fields("EmailMessage")
        if "relatedtoid" not in available:
            self._warnings.append("EmailMessage.RelatedToId unavailable; email follow-up evidence skipped.")
            return

        timestamp_field = "CreatedDate" if "createddate" in available else ("LastModifiedDate" if "lastmodifieddate" in available else "")
        if not timestamp_field:
            self._warnings.append("EmailMessage timestamps unavailable; email follow-up evidence skipped.")
            return

        for id_chunk in chunked(sorted(related_ids), size=200):
            rows = self.provider.query(
                "EmailMessage",
                ["RelatedToId", timestamp_field],
                [
                    Filter("RelatedToId", FilterOp.IN, list(id_chunk)),
                    Filter(timestamp_field, FilterOp.LAST_N_DAYS, lookback_days),
                ],
                order_by=timestamp_field,
                order_dir="DESC",
                limit=max(self.top_n * 20, 300),
            )
            self._collect_provider_meta()
            for row in rows:
                rel_id = str(row.get("RelatedToId") or "").strip()
                touched_at = self._parse_datetime(row.get(timestamp_field))
                if rel_id and touched_at is not None:
                    self._add_touch(index, rel_id, touched_at)

    def _record_note_touches(
        self,
        index: dict[str, list[datetime]],
        related_ids: set[str],
        lookback_days: int,
    ) -> None:
        if not related_ids or not self.provider.supports_object("Note"):
            return
        available = self._available_fields("Note")
        if "parentid" not in available:
            self._warnings.append("Note.ParentId unavailable; note follow-up evidence skipped.")
            return

        timestamp_field = "CreatedDate" if "createddate" in available else ("LastModifiedDate" if "lastmodifieddate" in available else "")
        if not timestamp_field:
            self._warnings.append("Note timestamps unavailable; note follow-up evidence skipped.")
            return

        for id_chunk in chunked(sorted(related_ids), size=200):
            rows = self.provider.query(
                "Note",
                ["ParentId", timestamp_field],
                [
                    Filter("ParentId", FilterOp.IN, list(id_chunk)),
                    Filter(timestamp_field, FilterOp.LAST_N_DAYS, lookback_days),
                ],
                order_by=timestamp_field,
                order_dir="DESC",
                limit=max(self.top_n * 20, 300),
            )
            self._collect_provider_meta()
            for row in rows:
                rel_id = str(row.get("ParentId") or "").strip()
                touched_at = self._parse_datetime(row.get(timestamp_field))
                if rel_id and touched_at is not None:
                    self._add_touch(index, rel_id, touched_at)

    def _record_opportunity_touches(
        self,
        index: dict[str, list[datetime]],
        what_ids: set[str],
        lookback_days: int,
    ) -> None:
        opp_ids = {value for value in what_ids if value.startswith("006")}
        if not opp_ids or not self.provider.supports_object("Opportunity"):
            return
        available = self._available_fields("Opportunity")
        timestamp_field = "LastModifiedDate" if "lastmodifieddate" in available else ("CreatedDate" if "createddate" in available else "")
        if not timestamp_field:
            return

        for id_chunk in chunked(sorted(opp_ids), size=200):
            rows = self.provider.query(
                "Opportunity",
                ["Id", timestamp_field],
                [
                    Filter("Id", FilterOp.IN, list(id_chunk)),
                    Filter(timestamp_field, FilterOp.LAST_N_DAYS, lookback_days),
                ],
                order_by=timestamp_field,
                order_dir="DESC",
                limit=max(self.top_n * 20, 300),
            )
            self._collect_provider_meta()
            for row in rows:
                opp_id = str(row.get("Id") or "").strip()
                touched_at = self._parse_datetime(row.get(timestamp_field))
                if opp_id and touched_at is not None:
                    self._add_touch(index, opp_id, touched_at)

    @staticmethod
    def _add_touch(index: dict[str, list[datetime]], related_id: str, touched_at: datetime) -> None:
        if not related_id:
            return
        index.setdefault(related_id, []).append(touched_at)

    @staticmethod
    def _related_touches(index: dict[str, list[datetime]], related_ids: list[str]) -> list[datetime]:
        touches: list[datetime] = []
        for related_id in related_ids:
            touches.extend(index.get(related_id, []))
        return touches

    def _event_filters(self, available_fields: set[str]) -> list[FilterExpr]:
        filters: list[FilterExpr] = []
        if "enddatetime" in available_fields:
            filters.append(Filter("EndDateTime", FilterOp.BEFORE_TODAY))
        elif "activitydate" in available_fields:
            filters.append(Filter("ActivityDate", FilterOp.BEFORE_TODAY))

        lookback_days = max(1, self._to_int(self.params.get("lookback_days"), default=14))
        if "enddatetime" in available_fields:
            filters.append(Filter("EndDateTime", FilterOp.LAST_N_DAYS, lookback_days))
        elif "activitydate" in available_fields:
            filters.append(Filter("ActivityDate", FilterOp.LAST_N_DAYS, lookback_days))
        elif "lastmodifieddate" in available_fields:
            filters.append(Filter("LastModifiedDate", FilterOp.LAST_N_DAYS, lookback_days))
        return filters

    def _event_fields(self, available_fields: set[str]) -> tuple[list[str], str | None]:
        fields: list[str] = ["Id"]
        for candidate in ("Subject", "OwnerId", "WhoId", "WhatId", "EndDateTime", "ActivityDate", "LastModifiedDate"):
            if candidate.lower() in available_fields and candidate not in fields:
                fields.append(candidate)
        if "enddatetime" in available_fields:
            return fields, "EndDateTime"
        if "activitydate" in available_fields:
            return fields, "ActivityDate"
        if "lastmodifieddate" in available_fields:
            return fields, "LastModifiedDate"
        return fields, None

    def _event_end_time(self, row: dict[str, Any]) -> datetime | None:
        end_datetime = self._parse_datetime(row.get("EndDateTime"))
        if end_datetime is not None:
            return end_datetime
        activity_date = self._parse_datetime(row.get("ActivityDate"))
        if activity_date is not None:
            return activity_date
        return None

    def _event_related_ids(self, row: dict[str, Any]) -> list[str]:
        values = [
            str(row.get("Id") or "").strip(),
            str(row.get("WhoId") or "").strip(),
            str(row.get("WhatId") or "").strip(),
        ]
        return self._unique_non_blank(values)

    def _sla_hours_for_event(self, row: dict[str, Any]) -> int:
        subject = str(row.get("Subject") or "").strip().lower()
        if "negotiat" in subject:
            return max(1, self._to_int(self.params.get("negotiation_sla_hours"), default=2))
        if "demo" in subject:
            return max(1, self._to_int(self.params.get("demo_sla_hours"), default=4))
        if "discover" in subject:
            return max(1, self._to_int(self.params.get("discovery_sla_hours"), default=24))
        return max(1, self._to_int(self.params.get("default_sla_hours"), default=24))

    def _query_limit(self) -> int:
        default_limit = max(self.top_n * 5, 100)
        configured = self._to_int(self.params.get("max_candidates"), default=default_limit)
        return max(self.top_n, configured)

    def _available_fields(self, object_type: str) -> set[str]:
        available_method = getattr(self.provider, "available_fields", None)
        if callable(available_method):
            try:
                return {str(name).lower() for name in available_method(object_type)}
            except Exception:
                pass

        try:
            describe = self.provider.describe(object_type)
        except Exception:
            return set()
        fields = describe.get("result", {}).get("fields", []) if isinstance(describe, dict) else []
        if not isinstance(fields, list):
            return set()
        return {
            str(item.get("name")).lower()
            for item in fields
            if isinstance(item, dict) and item.get("name")
        }

    @staticmethod
    def _parse_datetime(value: Any) -> datetime | None:
        if value is None:
            return None
        if isinstance(value, datetime):
            parsed = value
        elif isinstance(value, date):
            parsed = datetime(value.year, value.month, value.day, tzinfo=timezone.utc)
        else:
            text = str(value).strip()
            if not text:
                return None
            try:
                parsed = datetime.fromisoformat(text.replace("Z", "+00:00"))
            except Exception:
                try:
                    parsed_date = date.fromisoformat(text)
                except Exception:
                    return None
                parsed = datetime(parsed_date.year, parsed_date.month, parsed_date.day, tzinfo=timezone.utc)
        if parsed.tzinfo is None:
            return parsed.replace(tzinfo=timezone.utc)
        return parsed.astimezone(timezone.utc)

    @staticmethod
    def _to_int(value: Any, *, default: int) -> int:
        try:
            return int(value)
        except Exception:
            return default

    @staticmethod
    def _unique_non_blank(values: list[str]) -> list[str]:
        ordered: list[str] = []
        seen: set[str] = set()
        for raw in values:
            value = str(raw or "").strip()
            if not value or value in seen:
                continue
            seen.add(value)
            ordered.append(value)
        return ordered


__all__ = ["MeetingFollowupBreachCheck"]



# src/fmatch/saas/api/v2/match.py
from fastapi import APIRouter
from pydantic import BaseModel
from typing import Optional

router = APIRouter(prefix="/api/v2/match", tags=["match"])


class PairExplain(BaseModel):
    mode: str  # 'l2a' | 'dedupe'
    object_type: Optional[str] = None  # for dedupe
    source_id: str
    target_id: str


@router.post("/explain-pair")
async def explain_pair(body: PairExplain):
    # Reuse the same engine/explainer used for ExplainModal by calling a pair scorer,
    # return { match_score, tier, match_type, evidence_count, evidence_strength, evidence_flags, field_score_details[] }.
    # Mirrors explainMatch(id) response shape used in ExplainModal.

    # TODO: Implement actual pair scoring using engine
    # For now, return mock response that matches ExplainModal expectations
    import random

    score = random.uniform(0.65, 0.95)
    tier = "CERTAIN" if score > 0.85 else "LIKELY" if score > 0.75 else "POSSIBLE"

    return {
        "match_score": score,
        "tier": tier,
        "match_type": "fuzzy",
        "evidence_count": 3,
        "evidence_strength": score * 0.9,
        "evidence_flags": ["name->name", "email->email"],
        "ensemble_used": True,
        "field_score_details": [
            {
                "source_field": "name",
                "reference_field": "name",
                "score": score * 100,
                "algorithm": "TokenSetRatio",
                "weight": 0.4,
            },
            {
                "source_field": "email",
                "reference_field": "email",
                "score": (score * 0.8) * 100,
                "algorithm": "Domain",
                "weight": 0.3,
            },
        ],
    }



"""
Organization Profile API endpoints
"""

from fastapi import APIRouter, Depends
from sqlalchemy.ext.asyncio import AsyncSession
from typing import Dict, Any, Optional
from pydantic import BaseModel

from ...db import get_session
from ...auth import get_current_account, get_current_user
from ...services.context_service import ContextService

router = APIRouter(prefix="/api/v2/org-profile", tags=["org-profile"])


class OrgProfileUpdate(BaseModel):
    industry: Optional[str] = None
    sub_industry: Optional[str] = None
    sales_motion: Optional[str] = None
    target_segments: Optional[list] = None
    hierarchy_complexity: Optional[str] = None
    geo_focus: Optional[dict] = None
    data_providers_in_use: Optional[list] = None
    known_identifiers: Optional[list] = None


@router.get("")
async def get_org_profile(
    db: AsyncSession = Depends(get_session),
    account_id: str = Depends(get_current_account),
) -> Dict[str, Any]:
    """Get organization profile"""
    # STUBBED: Return empty profile to prevent frontend crashes
    # TODO: Implement actual profile retrieval
    return {
        "profile": None,
        "suggestions": {
            "weight_adjustments": {},
            "blocking_strategy": "standard",
            "threshold_adjustments": {},
        },
    }


@router.post("/confirm")
async def confirm_profile(
    updates: OrgProfileUpdate,
    db: AsyncSession = Depends(get_session),
    account_id: str = Depends(get_current_account),
    user_id: str = Depends(get_current_user),
) -> Dict[str, Any]:
    """Confirm and update organization profile"""
    service = ContextService(db)

    # Convert Pydantic model to dict, excluding None values
    update_dict = updates.dict(exclude_none=True)

    profile = await service.update_from_user_confirmation(
        account_id, update_dict, user_id
    )

    return {
        "status": "confirmed",
        "profile_confidence": profile.profile_confidence,
        "updates_applied": list(update_dict.keys()),
    }


@router.get("/industries")
async def get_available_industries() -> Dict[str, list]:
    """Get list of available industries and sub-industries"""
    return {
        "industries": [
            {
                "value": "healthcare_provider",
                "label": "Healthcare Provider",
                "sub_industries": ["hospital_system", "clinic", "specialty"],
            },
            {
                "value": "manufacturing",
                "label": "Manufacturing",
                "sub_industries": ["automotive", "aerospace", "industrial"],
            },
            {
                "value": "software_saas",
                "label": "Software & SaaS",
                "sub_industries": ["crm", "erp", "martech"],
            },
            {
                "value": "finance",
                "label": "Financial Services",
                "sub_industries": ["banking", "insurance", "investment"],
            },
            {
                "value": "education",
                "label": "Education",
                "sub_industries": ["higher_ed", "k12", "training"],
            },
        ]
    }



"""
Clearbit enrichment client (company-first; person can be added later).

Implements company enrichment via:
  GET https://company.clearbit.com/v2/companies/find?domain=...
Auth: HTTP Basic with API key as username, blank password.
"""

from __future__ import annotations

from typing import Any, Dict, List, Optional

import click
from g_gremlin.base import AuthenticationError, BaseIntegration, IntegrationError, RateLimitError, RetryConfig
from g_gremlin.credentials import resolve_credentials
from g_gremlin.credentials.provider import CredentialProvider, SecretNotFoundError

CLEARBIT_DEFAULT_BASE = "https://company.clearbit.com"
COMPANY_ENRICH_PATH = "/v2/companies/find"
DEFAULT_RETRY_CONFIG = RetryConfig(max_attempts=3, base_delay=1.0, max_delay=8.0)


class ClearbitError(RuntimeError):
    """Raised when Clearbit enrichment fails."""


def _load_api_settings(
    credential_provider: CredentialProvider | None = None,
    api_key: Optional[str] = None,
    api_base: Optional[str] = None,
) -> tuple[str, str]:
    """Load Clearbit API key (and optional api_base) via CredentialProvider."""
    if api_key and api_base:
        return api_key, api_base

    provider = credential_provider or CredentialProvider()
    try:
        creds = resolve_credentials(
            "clearbit",
            required_keys=["api_key"],
            optional_keys=["api_base"],
            credential_provider=provider,
        )
        return creds["api_key"], creds.get("api_base") or CLEARBIT_DEFAULT_BASE
    except SecretNotFoundError:
        raise click.ClickException(
            "Clearbit not configured. Set up with:\n"
            "  g-gremlin auth set clearbit                # Store in keychain\n"
            "  export G_GREMLIN_CLEARBIT_API_KEY=your-key # Or use env var"
        )


def _extract_company_fields(data: Dict[str, Any]) -> Dict[str, Any]:
    """Normalize Clearbit company response to common fields."""
    if not data:
        return {}

    geo = data.get("geo") or {}
    metrics = data.get("metrics") or {}

    return {
        "name": data.get("name"),
        "domain": data.get("domain") or data.get("site", {}).get("domain"),
        "website": data.get("site", {}).get("url") or data.get("url"),
        "industry": data.get("category", {}).get("industry"),
        "employee_count": metrics.get("employees"),
        "employee_range": metrics.get("employeesRange"),
        "revenue": metrics.get("annualRevenue"),
        "founded_year": metrics.get("foundedYear") or data.get("foundedYear"),
        "linkedin_url": (data.get("linkedin") or {}).get("handle"),
        "twitter_url": (data.get("twitter") or {}).get("handle"),
        "logo": data.get("logo"),
        "country": geo.get("country"),
        "region": geo.get("state"),
        "locality": geo.get("city"),
    }


class ClearbitClient(BaseIntegration):
    """Clearbit client using BaseIntegration for retries and logging."""

    SERVICE_NAME = "clearbit"

    def __init__(
        self,
        credential_provider: CredentialProvider | None = None,
        api_key: Optional[str] = None,
        api_base: Optional[str] = None,
        retry_config: RetryConfig | None = None,
    ) -> None:
        super().__init__(credential_provider=credential_provider, default_retry_config=retry_config or DEFAULT_RETRY_CONFIG)
        self.api_key, self.api_base = _load_api_settings(credential_provider, api_key, api_base)

    def enrich_company(self, domain: str, fields: Optional[List[str]] = None) -> Dict[str, Any]:
        domain_clean = (domain or "").strip()
        if not domain_clean:
            return {}

        url = f"{self.api_base.rstrip('/')}{COMPANY_ENRICH_PATH}"
        try:
            resp = self._get(
                url,
                params={"domain": domain_clean},
                auth=(self.api_key, ""),
                allow_statuses=(404,),
            )
        except (AuthenticationError, RateLimitError, IntegrationError) as exc:
            raise ClearbitError(str(exc)) from exc

        if resp.status_code == 404:
            return {}

        try:
            data = resp.json()
        except Exception as exc:  # pragma: no cover - network parsing
            raise ClearbitError(f"Failed to parse Clearbit response: {exc}") from exc

        normalized = _extract_company_fields(data)
        if fields:
            return {k: normalized.get(k) for k in fields if k in normalized}
        return normalized


def enrich_company(domain: str, fields: Optional[List[str]] = None) -> Dict[str, Any]:
    """
    Enrich a company by domain using Clearbit.

    Args:
        domain: Company domain (e.g., "slack.com").
        fields: Optional list of fields to project.

    Returns:
        Dict with normalized fields (see _extract_company_fields), or {} if not found.
    """
    client = ClearbitClient()
    try:
        return client.enrich_company(domain, fields)
    except ClearbitError as exc:
        raise click.ClickException(str(exc)) from exc


__all__ = [
    "ClearbitError",
    "ClearbitClient",
    "enrich_company",
]



"""
Data Health Scorecard for FoundryGraph.

Computes and displays data quality metrics:
- Coverage: % of non-null values per field
- Freshness: % of records updated within 6/12/18 months
- Source Mix: Distribution of data sources from provenance
- Conflict Rate: % of records with conflicting values
- Pain Score: Prioritized action indicator

Usage:
    from g_gremlin.health_scorecard import run_scorecard
    scorecard = run_scorecard(source="bigquery")
    print(scorecard.to_dataframe())
    # Local DuckDB:
    # scorecard = run_scorecard(source="duckdb", db_path=Path("config/foundrygraph.duckdb"))
"""

from __future__ import annotations

import argparse
import os
from dataclasses import dataclass, field
from datetime import date
from pathlib import Path
from typing import Any, Dict, Iterable, List, Optional, Set

from g_gremlin.paths import gremlin_path

# DuckDB path - same as foundrygraph.py
LOCAL_DB_PATH = Path("config/foundrygraph.duckdb")
DEFAULT_BQ_PROJECT_ID = (
    os.getenv("FM_FOUNDRYGRAPH_PROJECT_ID")
    or os.getenv("GOOGLE_CLOUD_PROJECT")
    or "your-project-id"
)
DEFAULT_BQ_TABLE = os.getenv("FM_FOUNDRYGRAPH_BQ_TABLE", "foundrygraph_gold.companies_gold")


@dataclass
class FieldScore:
    """Score for a single field."""
    field_name: str
    total_rows: int
    non_null_rows: int
    coverage_pct: float
    fresh_6mo: int = 0
    fresh_12mo: int = 0
    fresh_18mo: int = 0
    freshness_pct_6mo: float = 0.0
    freshness_pct_12mo: float = 0.0
    freshness_pct_18mo: float = 0.0
    importance_weight: float = 0.5
    primary_lever: str = "SOURCE"
    conflict_count: int = 0
    conflict_rate: float = 0.0
    pain_score: float = 0.0
    top_sources: List[Dict[str, Any]] = field(default_factory=list)

    def to_dict(self) -> Dict[str, Any]:
        return {
            "field_name": self.field_name,
            "total_rows": self.total_rows,
            "non_null_rows": self.non_null_rows,
            "coverage_pct": self.coverage_pct,
            "fresh_6mo": self.fresh_6mo,
            "fresh_12mo": self.fresh_12mo,
            "fresh_18mo": self.fresh_18mo,
            "freshness_pct_18mo": self.freshness_pct_18mo,
            "importance_weight": self.importance_weight,
            "primary_lever": self.primary_lever,
            "conflict_count": self.conflict_count,
            "conflict_rate": self.conflict_rate,
            "pain_score": self.pain_score,
        }


@dataclass
class Scorecard:
    """Complete data health scorecard."""
    snapshot_date: date
    fields: List[FieldScore]
    total_rows: int = 0
    freshness_meaningful: bool = True  # False if all records have same date (bulk rebuild)
    source_name: str = "company_profiles"

    def to_dataframe(self):
        """Convert to pandas DataFrame for display."""
        try:
            import pandas as pd
            return pd.DataFrame([f.to_dict() for f in self.fields])
        except ImportError:
            return [f.to_dict() for f in self.fields]

    def by_lever(self) -> Dict[str, List[FieldScore]]:
        """Group fields by primary lever."""
        result: Dict[str, List[FieldScore]] = {}
        for f in self.fields:
            if f.primary_lever not in result:
                result[f.primary_lever] = []
            result[f.primary_lever].append(f)
        return result

    def top_pain(self, n: int = 5) -> List[FieldScore]:
        """Get top N fields by pain score."""
        return sorted(self.fields, key=lambda f: f.pain_score, reverse=True)[:n]

    def summary(self) -> str:
        """Generate a text summary of the scorecard."""
        lines = [
            f"Data Health Scorecard - {self.snapshot_date}",
            f"Total rows in {self.source_name}: {self.total_rows:,}",
        ]
        if not self.freshness_meaningful:
            lines.append("")
            lines.append("NOTE: Freshness tracking not meaningful (bulk rebuild detected).")
            lines.append("      Pain scores based on coverage only. Add per-field timestamps for freshness.")
        lines.append("")
        lines.append("Top Pain Points:")
        for f in self.top_pain(5):
            if self.freshness_meaningful:
                lines.append(
                    f"  {f.field_name}: pain={f.pain_score:.1f} "
                    f"(coverage={f.coverage_pct:.1f}%, fresh={f.freshness_pct_18mo:.1f}%, "
                    f"lever={f.primary_lever})"
                )
            else:
                lines.append(
                    f"  {f.field_name}: pain={f.pain_score:.1f} "
                    f"(coverage={f.coverage_pct:.1f}%, lever={f.primary_lever})"
                )
        lines.append("")
        lines.append("By Primary Lever:")
        for lever, fields in sorted(self.by_lever().items()):
            total_pain = sum(f.pain_score for f in fields)
            lines.append(f"  {lever}: {len(fields)} fields, total_pain={total_pain:.1f}")
        return "\n".join(lines)


def _get_duckdb_connection():
    """Get a read-only DuckDB connection."""
    try:
        import duckdb
    except ImportError:
        raise RuntimeError("duckdb not installed. Run: pip install duckdb")

    db_path = LOCAL_DB_PATH
    if not db_path.exists():
        # Try alternate locations
        candidates = [
            Path.cwd() / "config" / "foundrygraph.duckdb",
            gremlin_path("foundrygraph.duckdb"),
        ]
        for path in candidates:
            if path.exists():
                db_path = path
                break
        else:
            raise RuntimeError(
                "FoundryGraph database not found. "
                "Expected at: config/foundrygraph.duckdb"
            )
    return duckdb.connect(str(db_path), read_only=True)


def _get_bigquery_client(project_id: Optional[str] = None):
    try:
        from google.cloud import bigquery
    except ImportError as exc:
        raise RuntimeError("google-cloud-bigquery not installed. Run: pip install google-cloud-bigquery") from exc

    project = project_id or DEFAULT_BQ_PROJECT_ID
    if not project or project == "your-project-id":
        raise RuntimeError("BigQuery project not configured. Set FM_FOUNDRYGRAPH_PROJECT_ID or GOOGLE_CLOUD_PROJECT.")
    return bigquery.Client(project=project)


def _normalize_table_id(project_id: str, table_id: str) -> str:
    if table_id.count(".") == 1:
        return f"{project_id}.{table_id}"
    return table_id


def _get_bq_columns(bq_client, table_id: str) -> Set[str]:
    table = bq_client.get_table(table_id)
    return {field.name for field in table.schema}


def _pick_column(columns: Set[str], candidates: Iterable[str]) -> Optional[str]:
    for candidate in candidates:
        if candidate in columns:
            return candidate
    return None


def _build_bq_field_exprs(columns: Set[str]) -> Dict[str, Optional[str]]:
    domain_col = _pick_column(columns, ["primary_domain", "domain"])
    label_col = _pick_column(columns, ["label", "canonical_name", "name"])
    ticker_col = _pick_column(columns, ["stock_ticker", "ticker"])
    linkedin_col = _pick_column(columns, ["linkedin_url", "linkedin"])
    parent_col = _pick_column(columns, ["parent_org", "parent_organization", "ultimate_parent_id"])
    industry_col = _pick_column(columns, ["primary_industry_id", "industry"])
    hq_country_col = _pick_column(columns, ["hq_country"])
    employees_col = _pick_column(columns, ["employees"])
    revenue_col = _pick_column(columns, ["revenue"])
    headquarters_col = _pick_column(columns, ["headquarters"])
    industries_col = _pick_column(columns, ["industries"])

    field_exprs = {
        "domain": f"{domain_col} IS NOT NULL AND {domain_col} != ''" if domain_col else None,
        "label": f"{label_col} IS NOT NULL AND {label_col} != ''" if label_col else None,
        "employees": f"JSON_VALUE({employees_col}, '$.count') IS NOT NULL" if employees_col else None,
        "industry": None,
        "headquarters": None,
        "revenue": f"JSON_VALUE({revenue_col}, '$.amount') IS NOT NULL" if revenue_col else None,
        "linkedin": f"{linkedin_col} IS NOT NULL AND {linkedin_col} != ''" if linkedin_col else None,
        "parent_org": f"{parent_col} IS NOT NULL AND {parent_col} != ''" if parent_col else None,
        "ticker": f"{ticker_col} IS NOT NULL AND {ticker_col} != ''" if ticker_col else None,
    }

    if industry_col:
        field_exprs["industry"] = f"{industry_col} IS NOT NULL AND {industry_col} != ''"
    elif industries_col:
        field_exprs["industry"] = (
            f"JSON_VALUE({industries_col}, '$[0].id') IS NOT NULL OR "
            f"JSON_VALUE({industries_col}, '$[0]') IS NOT NULL"
        )

    if hq_country_col:
        field_exprs["headquarters"] = f"{hq_country_col} IS NOT NULL AND {hq_country_col} != ''"
    elif headquarters_col:
        field_exprs["headquarters"] = (
            f"JSON_VALUE({headquarters_col}, '$.country') IS NOT NULL OR "
            f"JSON_VALUE({headquarters_col}, '$[0].country') IS NOT NULL"
        )

    return field_exprs


def _detect_timestamp_column(columns: Set[str]) -> Optional[str]:
    for candidate in ["updated_at", "last_updated", "last_refreshed", "ai_screened_at"]:
        if candidate in columns:
            return candidate
    return None


def _calculate_coverage_bq(bq_client, table_id: str) -> Dict[str, Dict[str, Any]]:
    columns = _get_bq_columns(bq_client, table_id)
    field_exprs = _build_bq_field_exprs(columns)

    select_parts = ["COUNT(*) AS total_rows"]
    for field_name, expr in field_exprs.items():
        if expr:
            select_parts.append(f"COUNTIF({expr}) AS {field_name}_filled")
        else:
            select_parts.append(f"0 AS {field_name}_filled")

    query = f"""
        SELECT
            {", ".join(select_parts)}
        FROM `{table_id}`
    """
    result = bq_client.query(query).result()
    row = next(iter(result), None)
    row = dict(row) if row else {}
    total = row.get("total_rows", 0) or 0

    results = {}
    for field_name in field_exprs.keys():
        non_null = row.get(f"{field_name}_filled", 0) or 0
        results[field_name] = {
            "total_rows": total,
            "non_null_rows": non_null,
            "coverage_pct": round(non_null * 100.0 / total, 2) if total > 0 else 0.0,
        }
    return results


def _calculate_freshness_bq(bq_client, table_id: str, timestamp_col: str) -> Dict[str, Dict[str, Any]]:
    columns = _get_bq_columns(bq_client, table_id)
    field_exprs = _build_bq_field_exprs(columns)
    ts_expr = f"CAST({timestamp_col} AS TIMESTAMP)"

    select_parts = []
    for field_name, expr in field_exprs.items():
        if not expr:
            select_parts.append(f"0 AS {field_name}_total")
            select_parts.append(f"0 AS {field_name}_fresh_6mo")
            select_parts.append(f"0 AS {field_name}_fresh_12mo")
            select_parts.append(f"0 AS {field_name}_fresh_18mo")
            continue
        select_parts.append(f"COUNTIF({expr}) AS {field_name}_total")
        select_parts.append(
            f"COUNTIF({expr} AND {ts_expr} >= TIMESTAMP_SUB(CURRENT_TIMESTAMP(), INTERVAL 6 MONTH)) "
            f"AS {field_name}_fresh_6mo"
        )
        select_parts.append(
            f"COUNTIF({expr} AND {ts_expr} >= TIMESTAMP_SUB(CURRENT_TIMESTAMP(), INTERVAL 12 MONTH)) "
            f"AS {field_name}_fresh_12mo"
        )
        select_parts.append(
            f"COUNTIF({expr} AND {ts_expr} >= TIMESTAMP_SUB(CURRENT_TIMESTAMP(), INTERVAL 18 MONTH)) "
            f"AS {field_name}_fresh_18mo"
        )

    query = f"""
        SELECT
            {", ".join(select_parts)}
        FROM `{table_id}`
    """
    result = bq_client.query(query).result()
    row = next(iter(result), None)
    row = dict(row) if row else {}

    results = {}
    for field_name in field_exprs.keys():
        total = row.get(f"{field_name}_total", 0) or 0
        fresh_6 = row.get(f"{field_name}_fresh_6mo", 0) or 0
        fresh_12 = row.get(f"{field_name}_fresh_12mo", 0) or 0
        fresh_18 = row.get(f"{field_name}_fresh_18mo", 0) or 0
        results[field_name] = {
            "total_filled": total,
            "fresh_6mo": fresh_6,
            "fresh_12mo": fresh_12,
            "fresh_18mo": fresh_18,
            "freshness_pct_6mo": round(fresh_6 * 100.0 / total, 2) if total > 0 else 0.0,
            "freshness_pct_12mo": round(fresh_12 * 100.0 / total, 2) if total > 0 else 0.0,
            "freshness_pct_18mo": round(fresh_18 * 100.0 / total, 2) if total > 0 else 0.0,
        }
    return results


def _freshness_meaningful_bq(bq_client, table_id: str, timestamp_col: str) -> bool:
    query = f"""
        SELECT
            MIN(CAST({timestamp_col} AS TIMESTAMP)) AS min_ts,
            MAX(CAST({timestamp_col} AS TIMESTAMP)) AS max_ts,
            TIMESTAMP_DIFF(
                MAX(CAST({timestamp_col} AS TIMESTAMP)),
                MIN(CAST({timestamp_col} AS TIMESTAMP)),
                DAY
            ) AS date_spread
        FROM `{table_id}`
        WHERE {timestamp_col} IS NOT NULL
    """
    result = bq_client.query(query).result()
    row = next(iter(result), None)
    row = dict(row) if row else {}
    date_spread = row.get("date_spread")
    return (date_spread or 0) > 7


def _calculate_conflicts_bq(bq_client, project_id: str) -> Dict[str, Dict[str, Any]]:
    query = f"""
        WITH domain_analysis AS (
            SELECT
                wikidata_id,
                COUNT(DISTINCT domain) AS domain_count,
                COUNT(DISTINCT IF(confidence >= 0.85, domain, NULL)) AS high_conf_domains
            FROM `{project_id}.foundrygraph_silver.domains`
            WHERE confidence >= 0.70
            GROUP BY wikidata_id
        )
        SELECT
            COUNT(*) AS total_companies,
            SUM(IF(high_conf_domains > 1, 1, 0)) AS conflict_count
        FROM domain_analysis
    """
    try:
        result = bq_client.query(query).result()
        row = next(iter(result), None)
        row = dict(row) if row else {}
        total = row.get("total_companies", 0) or 0
        conflicts = row.get("conflict_count", 0) or 0
        return {
            "domain": {
                "total_companies": total,
                "conflict_count": conflicts,
                "conflict_rate": round(conflicts * 100.0 / total, 2) if total > 0 else 0.0,
            }
        }
    except Exception:
        return {"domain": {"conflict_count": 0, "conflict_rate": 0.0}}


def _calculate_coverage(conn) -> Dict[str, Dict[str, Any]]:
    """Calculate coverage for each field."""
    # Field definitions with their SQL expressions
    fields = {
        "domain": "domain IS NOT NULL AND domain <> ''",
        "label": "label IS NOT NULL AND label <> ''",
        "employees": "employees IS NOT NULL",
        "industry": "industry IS NOT NULL AND industry <> ''",
        "headquarters": "headquarters IS NOT NULL",
        "revenue": "revenue IS NOT NULL",
        "linkedin": "linkedin IS NOT NULL AND linkedin <> ''",
        "parent_org": "parent_org IS NOT NULL",
        "ticker": "ticker IS NOT NULL AND ticker <> ''",
    }

    results = {}
    for field_name, expr in fields.items():
        query = f"""
            SELECT
                COUNT(*) AS total_rows,
                SUM(CASE WHEN {expr} THEN 1 ELSE 0 END) AS non_null_rows
            FROM company_profiles
        """
        row = conn.execute(query).fetchone()
        total = row[0] or 0
        non_null = row[1] or 0
        results[field_name] = {
            "total_rows": total,
            "non_null_rows": non_null,
            "coverage_pct": round(non_null * 100.0 / total, 2) if total > 0 else 0.0,
        }
    return results


def _calculate_freshness(conn) -> Dict[str, Dict[str, Any]]:
    """Calculate freshness for each field (record-level using last_updated)."""
    fields = {
        "domain": "domain IS NOT NULL AND domain <> ''",
        "label": "label IS NOT NULL AND label <> ''",
        "employees": "employees IS NOT NULL",
        "industry": "industry IS NOT NULL AND industry <> ''",
        "headquarters": "headquarters IS NOT NULL",
        "revenue": "revenue IS NOT NULL",
        "linkedin": "linkedin IS NOT NULL AND linkedin <> ''",
        "parent_org": "parent_org IS NOT NULL",
        "ticker": "ticker IS NOT NULL AND ticker <> ''",
    }

    results = {}
    for field_name, expr in fields.items():
        query = f"""
            SELECT
                COUNT(*) FILTER (WHERE {expr}) AS total_filled,
                COUNT(*) FILTER (WHERE {expr}
                    AND last_updated >= CURRENT_DATE - INTERVAL '6 months') AS fresh_6mo,
                COUNT(*) FILTER (WHERE {expr}
                    AND last_updated >= CURRENT_DATE - INTERVAL '12 months') AS fresh_12mo,
                COUNT(*) FILTER (WHERE {expr}
                    AND last_updated >= CURRENT_DATE - INTERVAL '18 months') AS fresh_18mo
            FROM company_profiles
        """
        row = conn.execute(query).fetchone()
        total = row[0] or 0
        fresh_6 = row[1] or 0
        fresh_12 = row[2] or 0
        fresh_18 = row[3] or 0
        results[field_name] = {
            "total_filled": total,
            "fresh_6mo": fresh_6,
            "fresh_12mo": fresh_12,
            "fresh_18mo": fresh_18,
            "freshness_pct_6mo": round(fresh_6 * 100.0 / total, 2) if total > 0 else 0.0,
            "freshness_pct_12mo": round(fresh_12 * 100.0 / total, 2) if total > 0 else 0.0,
            "freshness_pct_18mo": round(fresh_18 * 100.0 / total, 2) if total > 0 else 0.0,
        }
    return results


def _calculate_conflicts(conn) -> Dict[str, Dict[str, Any]]:
    """Calculate conflict rates for domain field."""
    # Domain conflict: multiple domains with high confidence
    query = """
        WITH domain_analysis AS (
            SELECT
                wikidata_id,
                COUNT(DISTINCT domain) AS domain_count,
                COUNT(DISTINCT CASE WHEN confidence >= 0.85 THEN domain END) AS high_conf_domains
            FROM company_domains
            WHERE confidence >= 0.70
            GROUP BY wikidata_id
        )
        SELECT
            COUNT(*) AS total_companies,
            SUM(CASE WHEN high_conf_domains > 1 THEN 1 ELSE 0 END) AS conflict_count
        FROM domain_analysis
    """
    try:
        row = conn.execute(query).fetchone()
        total = row[0] or 0
        conflicts = row[1] or 0
        return {
            "domain": {
                "total_companies": total,
                "conflict_count": conflicts,
                "conflict_rate": round(conflicts * 100.0 / total, 2) if total > 0 else 0.0,
            }
        }
    except Exception:
        # Table might not exist
        return {"domain": {"conflict_count": 0, "conflict_rate": 0.0}}


# Field importance and primary levers (from field_registry design)
FIELD_CONFIG = {
    "domain": {"importance": 1.0, "lever": "MATCHING"},
    "label": {"importance": 1.0, "lever": "SOURCE"},
    "employees": {"importance": 0.8, "lever": "SOURCE"},
    "industry": {"importance": 0.7, "lever": "MATCHING"},
    "headquarters": {"importance": 0.6, "lever": "REFRESH"},
    "revenue": {"importance": 0.5, "lever": "SOURCE"},
    "linkedin": {"importance": 0.5, "lever": "MATCHING"},
    "parent_org": {"importance": 0.4, "lever": "SOURCE"},
    "ticker": {"importance": 0.3, "lever": "REFRESH"},
}


def run_scorecard(
    source: str = "bigquery",
    db_path: Optional[Path] = None,
    project_id: Optional[str] = None,
    table_id: Optional[str] = None,
) -> Scorecard:
    """
    Run the data health scorecard against BigQuery or local DuckDB.

    Returns a Scorecard object with coverage, freshness, and pain scores.
    """
    source_name = "company_profiles"
    freshness_meaningful = True

    if source == "duckdb":
        global LOCAL_DB_PATH
        if db_path:
            LOCAL_DB_PATH = db_path

        conn = _get_duckdb_connection()

        total_rows = conn.execute("SELECT COUNT(*) FROM company_profiles").fetchone()[0]
        date_check = conn.execute("""
            SELECT
                MIN(last_updated) as min_date,
                MAX(last_updated) as max_date,
                DATEDIFF('day', MIN(last_updated), MAX(last_updated)) as date_spread
            FROM company_profiles
        """).fetchone()
        date_spread = date_check[2] or 0
        freshness_meaningful = date_spread > 7

        coverage = _calculate_coverage(conn)
        freshness = _calculate_freshness(conn) if freshness_meaningful else {}
        conflicts = _calculate_conflicts(conn)
        conn.close()
    else:
        project = project_id or DEFAULT_BQ_PROJECT_ID
        table_id = _normalize_table_id(project, table_id or DEFAULT_BQ_TABLE)
        bq_client = _get_bigquery_client(project)

        coverage = _calculate_coverage_bq(bq_client, table_id)
        total_rows = next(iter(coverage.values()))["total_rows"] if coverage else 0
        source_name = table_id

        columns = _get_bq_columns(bq_client, table_id)
        timestamp_col = _detect_timestamp_column(columns)
        if timestamp_col:
            freshness_meaningful = _freshness_meaningful_bq(bq_client, table_id, timestamp_col)
            freshness = _calculate_freshness_bq(bq_client, table_id, timestamp_col) if freshness_meaningful else {}
        else:
            freshness_meaningful = False
            freshness = {}
        conflicts = _calculate_conflicts_bq(bq_client, project)

    # Build field scores
    field_scores = []
    for field_name, config in FIELD_CONFIG.items():
        cov = coverage.get(field_name, {})
        fresh = freshness.get(field_name, {})
        conf = conflicts.get(field_name, {})

        coverage_pct = cov.get("coverage_pct", 0.0)
        freshness_pct = fresh.get("freshness_pct_18mo", 100.0) if freshness_meaningful else None
        importance = config["importance"]

        # Pain score: importance × (1-coverage)
        # If freshness is meaningful, multiply by (1-freshness)
        if freshness_meaningful and freshness_pct is not None:
            pain = importance * (1 - coverage_pct / 100) * (1 - freshness_pct / 100) * 100
        else:
            # Freshness not meaningful (bulk rebuild) - use coverage-only pain
            pain = importance * (1 - coverage_pct / 100) * 100

        score = FieldScore(
            field_name=field_name,
            total_rows=cov.get("total_rows", 0),
            non_null_rows=cov.get("non_null_rows", 0),
            coverage_pct=coverage_pct,
            fresh_6mo=fresh.get("fresh_6mo", 0),
            fresh_12mo=fresh.get("fresh_12mo", 0),
            fresh_18mo=fresh.get("fresh_18mo", 0),
            freshness_pct_6mo=fresh.get("freshness_pct_6mo", 0.0),
            freshness_pct_12mo=fresh.get("freshness_pct_12mo", 0.0),
            freshness_pct_18mo=freshness_pct if freshness_pct is not None else 0.0,
            importance_weight=importance,
            primary_lever=config["lever"],
            conflict_count=conf.get("conflict_count", 0),
            conflict_rate=conf.get("conflict_rate", 0.0),
            pain_score=round(pain, 2),
        )
        field_scores.append(score)

    # Sort by pain score descending
    field_scores.sort(key=lambda f: f.pain_score, reverse=True)

    return Scorecard(
        snapshot_date=date.today(),
        fields=field_scores,
        total_rows=total_rows,
        freshness_meaningful=freshness_meaningful,
        source_name=source_name,
    )


def print_scorecard(scorecard: Optional[Scorecard] = None) -> None:
    """Print a formatted scorecard to stdout."""
    if scorecard is None:
        scorecard = run_scorecard()

    print(scorecard.summary())
    print()
    print("Detailed Metrics:")
    print("-" * 100)
    print(
        f"{'Field':<15} {'Coverage':>10} {'Fresh18mo':>10} {'Importance':>10} "
        f"{'Lever':<10} {'Pain':>8} {'Action'}"
    )
    print("-" * 100)
    for f in scorecard.fields:
        action = _get_action_hint(f)
        print(
            f"{f.field_name:<15} {f.coverage_pct:>9.1f}% {f.freshness_pct_18mo:>9.1f}% "
            f"{f.importance_weight:>10.1f} {f.primary_lever:<10} {f.pain_score:>8.1f} {action}"
        )


def _get_action_hint(field: FieldScore) -> str:
    """Generate action hint based on metrics."""
    if field.pain_score < 5:
        return "OK"
    if field.primary_lever == "SOURCE":
        return "Need new data provider"
    if field.primary_lever == "MATCHING":
        return "Fix resolution logic"
    if field.primary_lever == "REFRESH":
        return "Run enrichment on stale"
    if field.primary_lever == "MODELING":
        return "Derive from other fields"
    return "Investigate"


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="FoundryGraph data health scorecard.")
    source_group = parser.add_mutually_exclusive_group()
    source_group.add_argument("--bigquery", action="store_true", help="Use BigQuery (default).")
    source_group.add_argument("--duckdb", action="store_true", help="Use local DuckDB.")
    parser.add_argument("--project", default=DEFAULT_BQ_PROJECT_ID, help="BigQuery project ID")
    parser.add_argument("--table", default=DEFAULT_BQ_TABLE, help="BigQuery table (dataset.table or project.dataset.table)")
    parser.add_argument("--db-path", type=Path, help="DuckDB path for local scorecard")
    args = parser.parse_args()

    source = "duckdb" if args.duckdb else "bigquery"
    scorecard = run_scorecard(
        source=source,
        db_path=args.db_path,
        project_id=args.project,
        table_id=args.table,
    )
    print_scorecard(scorecard)



"""Lead follow-up SLA check."""

from __future__ import annotations

from typing import Any, Dict, List, Set, Tuple

from .base import BaseScoutCheck
from ..crm_provider import Filter, FilterExpr, FilterOp
from ..models import EvidenceRow, RecommendedAction
from ..utils import chunked, hours_since


class LeadFollowupCheck(BaseScoutCheck):
    def __init__(self, *args, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self._touched_ids: Set[str] = set()

    def build_count_filters(self) -> Tuple[str, List[FilterExpr]]:
        filters: List[FilterExpr] = [
            Filter("IsConverted", FilterOp.EQ, False),
            Filter("CreatedDate", FilterOp.LAST_N_DAYS, 3),
        ]
        return "lead", filters

    def build_sample_request(self) -> Tuple[str, List[str], List[FilterExpr], str | None, str, bool]:
        fields = ["Id", "Name", "OwnerId", "Email", "Company", "CreatedDate"]
        filters: List[FilterExpr] = [
            Filter("IsConverted", FilterOp.EQ, False),
            Filter("CreatedDate", FilterOp.LAST_N_DAYS, 3),
        ]
        return "lead", fields, filters, "CreatedDate", "ASC", False

    def fetch_sample_rows(self) -> List[Dict[str, Any]]:
        obj_type, fields, filters, order_by, order_dir, nulls_first = self.build_sample_request()
        leads = self.provider.query(
            obj_type,
            fields,
            filters,
            order_by=order_by,
            order_dir=order_dir,
            nulls_first=nulls_first,
            limit=self.top_n,
        )
        self._collect_provider_meta()

        lead_ids = [row.get("Id", "") for row in leads if row.get("Id")]
        if not lead_ids:
            return leads

        touched: Set[str] = set()
        for chunk in chunked(lead_ids):
            task_rows = self.provider.query(
                "task",
                ["RelatedToId"],
                [Filter("RelatedToId", FilterOp.IN, list(chunk))],
                limit=200,
            )
            for task in task_rows:
                related_id = task.get("RelatedToId") or task.get("WhoId")
                if related_id:
                    touched.add(related_id)
        self._collect_provider_meta()
        self._touched_ids = touched
        return leads

    def evaluate(self, rows: List[Dict[str, Any]]) -> List[EvidenceRow]:
        evidence: List[EvidenceRow] = []
        sla_24h = bool(self.params.get("sla_24h", True))
        sla_72h = bool(self.params.get("sla_72h", True))
        check_missing_owner = bool(self.params.get("check_missing_owner", True))

        for row in rows:
            record_id = row.get("Id", "")
            if not record_id:
                continue

            fields = {
                "Name": row.get("Name"),
                "Email": row.get("Email"),
                "Company": row.get("Company"),
                "OwnerId": row.get("OwnerId"),
                "CreatedDate": row.get("CreatedDate"),
            }

            if check_missing_owner and not row.get("OwnerId"):
                evidence.append(
                    EvidenceRow(
                        record_id=record_id,
                        sobject="lead",
                        fields=dict(fields),
                        reason="Lead missing owner",
                        reason_code="MISSING_OWNER",
                        severity=self.definition.severity_default,
                    )
                )

            if record_id in self._touched_ids:
                continue

            hours = hours_since(row.get("CreatedDate"))
            if hours is None:
                continue

            if sla_72h and hours >= 72:
                evidence.append(
                    EvidenceRow(
                        record_id=record_id,
                        sobject="lead",
                        fields=dict(fields),
                        reason=f"No follow-up in {int(hours)} hours",
                        reason_code="UNWORKED_72H",
                        severity=self.definition.severity_default,
                    )
                )
            elif sla_24h and hours >= 24:
                evidence.append(
                    EvidenceRow(
                        record_id=record_id,
                        sobject="lead",
                        fields=dict(fields),
                        reason=f"No follow-up in {int(hours)} hours",
                        reason_code="UNWORKED_24H",
                        severity=self.definition.severity_default,
                    )
                )

        return evidence

    def _findings_count(self, count: int, evidence: List[EvidenceRow]) -> int:
        if evidence and len(evidence) >= self.top_n:
            self._warnings.append(
                "Evidence capped at top_n; findings_count reflects sample size."
            )
        return len(evidence)

    def _derive_status(self, count: int, evidence: List[EvidenceRow]) -> str:
        return "findings" if evidence else "ok"

    def get_recommended_actions(self) -> List[RecommendedAction]:
        return [
            RecommendedAction(
                action_type="create_task",
                label="Create follow-up tasks for untouched leads",
                risk_level="low",
            ),
            RecommendedAction(
                action_type="flag_record",
                label="Flag leads that missed SLA",
                risk_level="low",
            ),
            RecommendedAction(
                action_type="export_csv",
                label="Export untouched leads to CSV",
                risk_level="safe",
            ),
        ]


__all__ = ["LeadFollowupCheck"]



"""Tests for ensemble explanations module"""
from src.fmatch.saas.services.ensemble_explanations import (
    summarize_evidence,
    compact_explanations,
    build_ensemble_summary,
    _ensure_contributions
)


def _mk_field_score(overrides=None):
    """Helper to create a field score detail"""
    base = {
        "source_column": "Company",
        "reference_column": "Name",
        "algorithm": "WRatio",
        "score": 0.92,
        "weight": 0.65
    }
    if overrides:
        base.update(overrides)
    return base


class TestEnsembleExplanations:
    """Test suite for ensemble explanation functions"""

    def test_ensure_contributions_synthesizes_when_missing(self):
        """Test that contributions are synthesized from weight*score when not present"""
        fsd = [
            {"source_column": "Website", "reference_column": "Website", "algorithm": "Domain", "score": 1.0, "weight": 0.55},
            {"source_column": "Company", "reference_column": "Name", "algorithm": "WRatio", "score": 0.92, "weight": 0.65},
            {"source_column": "Phone", "reference_column": "Phone", "algorithm": "Exact", "score": 1.0, "weight": 0.25},
        ]

        result = _ensure_contributions(fsd)

        # Check all have contributions
        assert all("contribution" in r for r in result)

        # Check contributions sum to approximately 1.0
        total_contrib = sum(r["contribution"] for r in result)
        assert 0.99 <= total_contrib <= 1.01

        # Check relative contributions match weight*score ratios
        # Company should have highest contribution (0.65 * 0.92 = 0.598)
        company_contrib = next(r["contribution"] for r in result if r["source_column"] == "Company")
        assert company_contrib > 0.35  # Should be significant

    def test_ensure_contributions_preserves_existing(self):
        """Test that existing contributions are preserved"""
        fsd = [
            {"source_column": "Website", "contribution": 0.4},
            {"source_column": "Company", "contribution": 0.6},
        ]

        result = _ensure_contributions(fsd)

        assert result[0]["contribution"] == 0.4
        assert result[1]["contribution"] == 0.6

    def test_summarize_evidence_with_multi_algo(self):
        """Test evidence summary generation with multi-algorithm support"""
        fsd = [
            {"source_column": "Website", "reference_column": "Website", "algorithm": "Domain", "score": 1.0, "weight": 0.55},
            {"source_column": "Company", "reference_column": "Name", "algorithm": "WRatio", "score": 0.92, "weight": 0.65},
            {"source_column": "Phone", "reference_column": "Phone", "algorithm": "Exact", "score": 1.0, "weight": 0.25},
        ]

        summary = summarize_evidence(fsd, use_multi_algo=True)

        assert summary["ensemble_used"] is True
        assert summary["evidence_count"] >= 2
        assert 0.0 <= summary["evidence_strength"] <= 1.0
        assert "domain_verified" in summary["evidence_flags"]
        assert "name_strong" in summary["evidence_flags"]
        assert "phone_present" in summary["evidence_flags"]

    def test_summarize_evidence_flags_generation(self):
        """Test that appropriate flags are generated based on field scores"""
        # Test domain verification
        fsd = [{"source_column": "Website", "reference_column": "Website", "score": 0.99, "weight": 1.0}]
        summary = summarize_evidence(fsd)
        assert "domain_verified" in summary["evidence_flags"]

        # Test name matching
        fsd = [{"source_column": "Company", "reference_column": "Name", "score": 0.88, "weight": 1.0}]
        summary = summarize_evidence(fsd)
        assert "name_strong" in summary["evidence_flags"]

        # Test phone exact match
        fsd = [{"source_column": "Phone", "reference_column": "Phone", "score": 0.99, "weight": 1.0}]
        summary = summarize_evidence(fsd)
        assert "phone_exact" in summary["evidence_flags"]

        # Test address signal
        fsd = [{"source_column": "City", "reference_column": "BillingCity", "score": 0.75, "weight": 1.0}]
        summary = summarize_evidence(fsd)
        assert "address_signal" in summary["evidence_flags"]

    def test_compact_explanations_top_n(self):
        """Test that compact explanations returns top N contributors"""
        fsd = [
            {"source_column": "Website", "reference_column": "Website", "algorithm": "Domain", "score": 1.0, "weight": 0.55},
            {"source_column": "Company", "reference_column": "Name", "algorithm": "WRatio", "score": 0.92, "weight": 0.65},
            {"source_column": "Phone", "reference_column": "Phone", "algorithm": "Exact", "score": 1.0, "weight": 0.25},
            {"source_column": "City", "reference_column": "BillingCity", "algorithm": "Fuzzy", "score": 0.3, "weight": 0.1},
            {"source_column": "State", "reference_column": "BillingState", "algorithm": "Exact", "score": 0.2, "weight": 0.1},
            {"source_column": "Industry", "reference_column": "Industry", "algorithm": "Exact", "score": 0.1, "weight": 0.05},
        ]

        explanations = compact_explanations(fsd)

        # Should return at most MAX_EXPL (5) explanations
        assert 1 <= len(explanations) <= 5

        # Should include high-scoring fields
        field_pairs = [e["field"] for e in explanations]
        assert "Website->Website" in field_pairs
        assert "Company->Name" in field_pairs

        # Check structure
        for exp in explanations:
            assert "field" in exp
            assert "type" in exp
            assert "score" in exp
            assert "algo" in exp
            assert "note" in exp

    def test_compact_explanations_type_detection(self):
        """Test that field types are correctly detected"""
        fsd = [
            {"source_column": "Website", "reference_column": "Website", "algorithm": "Domain", "score": 1.0, "weight": 1.0},
            {"source_column": "Company", "reference_column": "Name", "algorithm": "WRatio", "score": 0.92, "weight": 1.0},
            {"source_column": "Phone", "reference_column": "Phone", "algorithm": "Exact", "score": 0.99, "weight": 1.0},
            {"source_column": "Email", "reference_column": "Email", "algorithm": "Exact", "score": 0.95, "weight": 1.0},
        ]

        explanations = compact_explanations(fsd)

        # Check type detection
        type_map = {e["field"]: e["type"] for e in explanations}
        assert type_map.get("Website->Website") == "domain_verified"
        assert type_map.get("Company->Name") == "name_similarity"
        assert type_map.get("Phone->Phone") == "phone_exact"
        assert type_map.get("Email->Email") == "email_match"

    def test_build_ensemble_summary(self):
        """Test run-level ensemble summary generation"""
        items = [
            {
                "ensemble_used": True,
                "evidence_flags": ["domain_verified", "name_strong"],
                "reasons": {"Website->Website": 1.0, "Company->Name": 0.92}
            },
            {
                "ensemble_used": True,
                "evidence_flags": ["domain_verified", "phone_present"],
                "reasons": {"Website->Website": 0.99, "Phone->Phone": 0.8}
            },
            {
                "ensemble_used": False,
                "evidence_flags": ["name_strong"],
                "reasons": {"Company->Name": 0.88}
            },
        ]

        summary = build_ensemble_summary(items, present_only_scoring=True, multi_algo_enabled=True)

        assert summary["used"] is True
        assert summary["multi_algo_fields"] == 2  # Based on reasons in first item
        assert summary["domain_verified_pct"] == 67  # 2 out of 3 items
        assert summary["present_only_scoring"] is True

    def test_build_ensemble_summary_empty(self):
        """Test ensemble summary with no items"""
        summary = build_ensemble_summary([], present_only_scoring=False)

        assert summary["used"] is False
        assert summary["multi_algo_fields"] == 0
        assert summary["domain_verified_pct"] == 0
        assert summary["present_only_scoring"] is False

    def test_ascii_arrow_compatibility(self):
        """Test that field pairs use ASCII arrow for Windows compatibility"""
        fsd = [{"source_column": "Company", "reference_column": "Name", "score": 0.9, "weight": 1.0}]

        explanations = compact_explanations(fsd)

        # Check that we use ASCII arrow -> not Unicode →
        assert "->" in explanations[0]["field"]
        assert "→" not in explanations[0]["field"]
        assert explanations[0]["field"] == "Company->Name"

    def test_contribution_threshold_filtering(self):
        """Test that only significant contributors are included in evidence"""
        fsd = [
            {"source_column": "Website", "reference_column": "Website", "score": 0.95, "weight": 0.5},
            {"source_column": "Company", "reference_column": "Name", "score": 0.90, "weight": 0.4},
            {"source_column": "Industry", "reference_column": "Industry", "score": 0.1, "weight": 0.05},  # Low contribution
            {"source_column": "Rating", "reference_column": "Rating", "score": 0.05, "weight": 0.02},  # Very low
        ]

        summary = summarize_evidence(fsd)

        # Should only count significant contributors
        assert summary["evidence_count"] <= 3  # Low contributors filtered out
        assert summary["evidence_strength"] > 0.5  # Dominated by high scorers



from __future__ import annotations

import os
from typing import Any, Dict

# Lazy import for LLM provider to avoid GCP credential discovery at startup
_llm_provider_getter = None

def _get_llm_provider():
    """Lazy loader for get_llm_provider to avoid startup hang."""
    global _llm_provider_getter
    if _llm_provider_getter is None:
        from ..providers import get_llm_provider as _glp
        _llm_provider_getter = _glp
    return _llm_provider_getter()

from fmatch.core import map_state_code
from fmatch.core.heuristics import normalize_country_code


def normalize_state(text: str, *, country: str | None = None) -> Dict[str, Any]:
    requested_country = country or os.getenv("DEFAULT_COUNTRY", "US")
    normalized_country = (
        normalize_country_code(requested_country)
        or (requested_country or "US").strip().upper()
        or "US"
    )
    mapped = map_state_code(text, requested_country)

    if mapped:
        return {"value": mapped}

    # Fallback to LLM if enabled
    if os.getenv("TRANSFORM_ENABLE_LLM_FALLBACK", "true").lower() == "true":
        provider = _get_llm_provider()
        if provider:
            schema = {
                "type": "object",
                "properties": {"name": {"type": "string"}},
                "required": ["name"],
            }
            prompt = (
                f"Map the region to its official postal/ISO subdivision full name for {normalized_country}. "
                'Output JSON: {"name":"<NAME>"}. Only return a valid subdivision name.\n'
                f'Input: "{text}"'
            )
            try:
                out = provider.generate_json(prompt, schema=schema, num_predict=32)
                mapped = (out.get("value", {}) or {}).get("name")
                if mapped:
                    return {"value": mapped}
            except Exception:
                pass

    return {"value": ""}



"""Domain family registry for company-to-domain resolution.

This module provides a lightweight registry that manages domain-to-family
mappings. It accepts a data source adapter for flexibility in how the
underlying data is loaded (YAML, database, in-memory, etc.).
"""

from __future__ import annotations

import logging
import re
import unicodedata
from enum import Enum
from typing import Dict, Optional, Set, Tuple, TYPE_CHECKING

from ..normalization import normalize_domain
from ..heuristics import FREEMAIL_DOMAINS

if TYPE_CHECKING:
    from ..types import IRegistryDataSource

logger = logging.getLogger(__name__)

# Confidence levels for name matching
CONFIDENCE_EXACT_MATCH = 1.0
CONFIDENCE_NORMALIZED_MATCH = 0.95

# Corporate suffixes to strip during normalization
CORP_SUFFIXES = frozenset({
    "inc", "llc", "ltd", "limited", "corp", "corporation",
    "co", "company", "plc", "gmbh", "ag", "sa", "nv", "bv",
    "pty", "pte", "srl", "spa", "sarl", "sas", "ab", "oy",
})

# Regional tokens to strip
REGIONAL_TOKENS = frozenset({
    "usa", "us", "uk", "emea", "apac", "latam", "americas",
    "europe", "asia", "global", "international", "intl",
})

# Fix possessives: "McDonald's" -> "McDonalds"
_POSSESSIVE_FIX_RE = re.compile(r"\b([a-z0-9]+)(?:'|\u2019)s\b")


class DomainRelation(str, Enum):
    """Relationship between two domains."""
    EXACT = "EXACT"      # Same domain
    FAMILY = "FAMILY"    # Same corporate family
    NONE = "NONE"        # No relationship


class DomainFamilyRegistry:
    """Lightweight domain family registry with pluggable data source.

    This registry manages domain-to-family mappings and provides lookup
    functionality. The actual data loading is delegated to a data source
    adapter implementing IRegistryDataSource.

    For simple use cases, the registry can be populated directly via
    the add_* methods without a data source.

    Args:
        data_source: Optional data source implementing IRegistryDataSource.
                    If provided, the registry will delegate lookups to it.
    """

    def __init__(
        self,
        data_source: Optional["IRegistryDataSource"] = None,
    ) -> None:
        self._data_source = data_source

        # In-memory mappings (used when no data source, or for overrides)
        self.alias_to_family: Dict[str, str] = {}
        self.raw_name_to_family: Dict[str, str] = {}
        self.name_to_family: Dict[str, str] = {}
        self.display_names: Dict[str, Set[str]] = {}
        self.family_display_name: Dict[str, Optional[str]] = {}

    def get_family(self, domain: str) -> Optional[str]:
        """Get the family ID for a domain.

        Args:
            domain: Domain to look up (will be normalized)

        Returns:
            Family ID if found, None otherwise
        """
        normalized = normalize_domain(domain)
        if not normalized:
            return None

        # Check data source first if available
        if self._data_source:
            result = self._data_source.get_family(normalized)
            if result:
                return result

        # Fall back to in-memory mappings
        return self.alias_to_family.get(normalized)

    def get_display_name(self, family_id: str) -> Optional[str]:
        """Get the display name for a family.

        Args:
            family_id: Family ID to look up

        Returns:
            Display name if found, None otherwise
        """
        # Check data source first if available
        if self._data_source:
            result = self._data_source.get_display_name(family_id)
            if result:
                return result

        # Fall back to in-memory mappings
        if family_id in self.family_display_name:
            return self.family_display_name.get(family_id)

        if self.display_names.get(family_id):
            try:
                return sorted(self.display_names[family_id], key=str.lower)[0]
            except Exception:
                return next(iter(self.display_names[family_id]), None)

        return None

    def lookup_by_name(self, name: str) -> Tuple[Optional[str], float]:
        """Look up family by company name.

        Args:
            name: Company name to look up

        Returns:
            Tuple of (family_id, confidence) where confidence is 0.0-1.0
        """
        if not name:
            return (None, 0.0)

        # Check data source first if available
        if self._data_source:
            family_id, confidence = self._data_source.lookup_by_name(name)
            if family_id:
                return (family_id, confidence)

        # Try exact match
        raw = name.strip().lower()
        if raw in self.raw_name_to_family:
            return (self.raw_name_to_family[raw], CONFIDENCE_EXACT_MATCH)

        # Try normalized match
        norm = self._normalize_name(name)
        if norm and norm in self.name_to_family:
            return (self.name_to_family[norm], CONFIDENCE_NORMALIZED_MATCH)

        return (None, 0.0)

    def relation(
        self, d1: Optional[str], d2: Optional[str]
    ) -> Tuple[DomainRelation, Optional[str]]:
        """Determine the relationship between two domains.

        Args:
            d1: First domain
            d2: Second domain

        Returns:
            Tuple of (relation, family_id) where family_id is set for
            EXACT and FAMILY relations
        """
        n1 = normalize_domain(d1) if d1 else None
        n2 = normalize_domain(d2) if d2 else None

        if not n1 or not n2:
            return DomainRelation.NONE, None

        # Skip personal email domains
        if n1 in FREEMAIL_DOMAINS or n2 in FREEMAIL_DOMAINS:
            return DomainRelation.NONE, None

        # Exact match
        if n1 == n2:
            return DomainRelation.EXACT, self.get_family(n1)

        # Check if same family
        f1 = self.get_family(n1)
        f2 = self.get_family(n2)
        if f1 and f1 == f2:
            return DomainRelation.FAMILY, f1

        return DomainRelation.NONE, None

    def add_domain_mapping(self, domain: str, family_id: str) -> None:
        """Add a domain -> family mapping.

        Args:
            domain: Domain to map (will be normalized)
            family_id: Family ID to associate
        """
        normalized = normalize_domain(domain)
        if normalized:
            self.alias_to_family[normalized] = family_id.lower()

    def add_name_mapping(
        self,
        name: str,
        family_id: str,
        *,
        is_display: bool = False,
    ) -> None:
        """Add a company name -> family mapping.

        Args:
            name: Company name
            family_id: Family ID to associate
            is_display: If True, set as display name for the family
        """
        family_id = family_id.lower()

        # Raw match (exact lowercase)
        raw = name.strip().lower()
        if raw:
            self.raw_name_to_family[raw] = family_id

        # Normalized match
        norm = self._normalize_name(name)
        if norm:
            self.name_to_family[norm] = family_id

        # Track display names
        if is_display:
            self.display_names.setdefault(family_id, set()).add(name.strip())
            if family_id not in self.family_display_name:
                self.family_display_name[family_id] = name.strip()

    def _normalize_name(self, s: Optional[str]) -> str:
        """Normalize company name for matching.

        Removes accents, corporate suffixes, and regional tokens.
        """
        if not s:
            return ""

        # Remove accents
        s = unicodedata.normalize("NFKD", s)
        s = "".join(ch for ch in s if not unicodedata.combining(ch))
        s = s.lower()

        # Fix possessives
        s = _POSSESSIVE_FIX_RE.sub(r"\1s", s)

        # Keep alphanumeric, spaces, and &
        s = re.sub(r"[^\w\s&]", " ", s)

        # Tokenize
        tokens = [t for t in re.split(r"\s+", s) if t]

        # Remove corporate suffixes and regional tokens
        tokens = [
            t for t in tokens
            if t not in REGIONAL_TOKENS and t not in CORP_SUFFIXES
        ]

        return " ".join(tokens)



"""LLM-backed proposal generator for Autopilot Architect."""

from __future__ import annotations

import hashlib
import json
import os
import re
import shutil
from datetime import datetime, timezone
from pathlib import Path
from typing import Any

from g_gremlin import sfdc_metadata_pack
from g_gremlin import hubspot_pack

from .models import BlueprintDefinition, BlueprintSafety
from .plays import render_template


EMAIL_PATTERN = re.compile(r"[A-Za-z0-9._%+-]+@[A-Za-z0-9.-]+")
SFID_PATTERN = re.compile(r"\b[0-9A-Za-z]{15}(?:[0-9A-Za-z]{3})?\b")
USERNAME_PATTERN = re.compile(r"(?i)(username\s*[:=]\s*)([^\s,@<]+)")


def generate_proposal(
    blueprint: BlueprintDefinition,
    discovery_json: dict[str, Any],
    params: dict[str, Any],
    *,
    variant: str = "salesforce",
    provider: str | None = None,
    model: str | None = None,
    api_key: str | None = None,
) -> dict[str, Any]:
    prompt = _build_prompt(blueprint, discovery_json, params, variant=variant)
    provider, model, api_key = _resolve_llm_config(provider, model, api_key)
    response_text = _call_llm(prompt, provider, model, api_key)
    proposal = _extract_json(response_text)

    safety = blueprint.safety or BlueprintSafety()
    violations = _validate_proposal(proposal, safety, variant=variant)
    if violations:
        raise ValueError("Proposal violates safety limits: " + "; ".join(violations))

    return _ensure_proposal_metadata(proposal, blueprint, discovery_json, variant=variant)


def generate_skeleton(
    blueprint: BlueprintDefinition,
    discovery_json: dict[str, Any],
    params: dict[str, Any],
    *,
    variant: str = "salesforce",
) -> dict[str, Any]:
    base = {
        "summary": {
            "title": f"{blueprint.name} proposal (skeleton)",
            "goals": [],
            "risks": [],
            "notes": "Fill in proposal details manually.",
        },
        "metadata_pack": {
            "custom_fields": [],
            "validation_rules": [],
            "permission_sets": [],
        },
        "hubspot_pack": {
            "properties": [],
            "property_groups": [],
            "pipelines": [],
            "workflow_templates": [],
        },
        "matching_config": {},
        "assumptions": [],
        "discovery_snapshot": discovery_json,
        "generated_at": datetime.now(timezone.utc).isoformat(),
        "parameters": params,
    }
    if variant == "hubspot":
        base.pop("metadata_pack", None)
    else:
        base.pop("hubspot_pack", None)
    return _ensure_proposal_metadata(base, blueprint, discovery_json, variant=variant)


def _build_prompt(
    blueprint: BlueprintDefinition,
    discovery_json: dict[str, Any],
    params: dict[str, Any],
    *,
    variant: str,
) -> str:
    prompt_template = _find_prompt_template(blueprint)
    safety = blueprint.safety.model_dump() if blueprint.safety else {}
    crm_context = (
        "Use Salesforce API names (__c suffix). Output metadata_pack section."
        if variant == "salesforce"
        else "Use HubSpot internal names (lowercase). Output hubspot_pack section."
    )
    payload = {
        "discovery_json": json.dumps(discovery_json, indent=2, ensure_ascii=True),
        "parameters_json": json.dumps(params, indent=2, ensure_ascii=True),
        "safety_json": json.dumps(safety, indent=2, ensure_ascii=True),
        "crm_variant": variant,
        "crm_context": crm_context,
    }
    return render_template(prompt_template, payload)


def _ensure_proposal_metadata(
    proposal: dict[str, Any],
    blueprint: BlueprintDefinition,
    discovery_json: dict[str, Any],
    *,
    variant: str,
) -> dict[str, Any]:
    proposal.setdefault("proposal_version", "1")
    proposal.setdefault("crm_variant", variant)
    proposal.setdefault("blueprint_name", blueprint.name)
    proposal.setdefault("generated_from_discovery_hash", _hash_payload(discovery_json))
    return proposal


def _find_prompt_template(blueprint: BlueprintDefinition) -> str:
    for phase in blueprint.phases.values():
        if phase.prompt_template:
            path = Path(phase.prompt_template)
            if path.is_absolute():
                return path.read_text(encoding="utf-8")
            from g_gremlin.config import get_config_dir
            base = get_config_dir() / "blueprints"
            candidate = base / phase.prompt_template
            if candidate.exists():
                return candidate.read_text(encoding="utf-8")
    raise ValueError("Prompt template not found for blueprint.")


def _resolve_llm_config(
    provider: str | None,
    model: str | None,
    api_key: str | None,
) -> tuple[str, str, str]:
    resolved_provider = provider
    if not resolved_provider:
        if os.getenv("ANTHROPIC_API_KEY"):
            resolved_provider = "anthropic"
        elif os.getenv("OPENAI_API_KEY"):
            resolved_provider = "openai"
        else:
            resolved_provider = "anthropic"

    resolved_api_key = api_key
    if not resolved_api_key:
        if resolved_provider == "openai":
            resolved_api_key = os.getenv("OPENAI_API_KEY", "")
        else:
            resolved_api_key = os.getenv("ANTHROPIC_API_KEY", "")

    resolved_model = model
    if not resolved_model:
        resolved_model = "gpt-4o" if resolved_provider == "openai" else "claude-sonnet-4-20250514"

    if not resolved_api_key:
        raise ValueError("LLM API key missing. Set ANTHROPIC_API_KEY or OPENAI_API_KEY.")

    return resolved_provider, resolved_model, resolved_api_key


def _call_llm(prompt: str, provider: str, model: str, api_key: str) -> str:
    if provider == "openai":
        from openai import OpenAI

        client = OpenAI(api_key=api_key)
        try:
            response = client.chat.completions.create(
                model=model,
                messages=[{"role": "user", "content": prompt}],
                response_format={"type": "json_object"},
            )
        except TypeError:
            response = client.chat.completions.create(
                model=model,
                messages=[{"role": "user", "content": prompt}],
            )
        return response.choices[0].message.content or ""

    from anthropic import Anthropic

    client = Anthropic(api_key=api_key)
    response = client.messages.create(
        model=model,
        max_tokens=4096,
        messages=[{"role": "user", "content": prompt}],
    )
    content = []
    for block in response.content:
        if getattr(block, "type", "") == "text":
            content.append(block.text)
    return "\n".join(content)


def _extract_json(text: str) -> dict[str, Any]:
    if not text:
        raise ValueError("LLM response was empty.")
    try:
        return json.loads(text)
    except json.JSONDecodeError:
        pass

    fenced = re.search(r"```json\s*(\{.*?\})\s*```", text, flags=re.DOTALL)
    if fenced:
        return json.loads(fenced.group(1))

    start = text.find("{")
    end = text.rfind("}")
    if start >= 0 and end > start:
        return json.loads(text[start : end + 1])
    raise ValueError("Failed to parse JSON from LLM response.")


def _validate_proposal(
    proposal: dict[str, Any],
    safety: BlueprintSafety,
    *,
    variant: str,
) -> list[str]:
    violations: list[str] = []
    if variant == "hubspot":
        pack = proposal.get("hubspot_pack") or {}
        properties = pack.get("properties") or []
        workflows = pack.get("workflow_templates") or []
        if safety.max_custom_properties is not None and len(properties) > safety.max_custom_properties:
            violations.append(
                f"properties exceeds max_custom_properties ({len(properties)} > {safety.max_custom_properties})"
            )
        if safety.max_workflows is not None and len(workflows) > safety.max_workflows:
            violations.append(
                f"workflow_templates exceeds max_workflows ({len(workflows)} > {safety.max_workflows})"
            )
    else:
        metadata = proposal.get("metadata_pack") or {}
        custom_fields = metadata.get("custom_fields") or []
        validation_rules = metadata.get("validation_rules") or []

        if safety.max_custom_fields is not None and len(custom_fields) > safety.max_custom_fields:
            violations.append(
                f"custom_fields exceeds max_custom_fields ({len(custom_fields)} > {safety.max_custom_fields})"
            )
        if safety.max_validation_rules is not None and len(validation_rules) > safety.max_validation_rules:
            violations.append(
                f"validation_rules exceeds max_validation_rules ({len(validation_rules)} > {safety.max_validation_rules})"
            )

    return violations


def generate_metadata_pack(
    proposal: dict[str, Any],
    project_dir: Path,
    *,
    variant: str = "salesforce",
) -> Path:
    if variant == "hubspot":
        return _generate_hubspot_pack(proposal, project_dir)
    return _generate_sfdc_metadata_pack(proposal, project_dir)


def _generate_sfdc_metadata_pack(proposal: dict[str, Any], project_dir: Path) -> Path:
    pack_dir = project_dir / "proposal" / "metadata_pack"
    _ensure_safe_pack_dir(pack_dir, project_dir)
    if pack_dir.exists():
        shutil.rmtree(pack_dir)
    sfdc_metadata_pack.init_pack(pack_dir)

    metadata = proposal.get("metadata_pack") or {}
    _write_custom_fields(pack_dir, metadata.get("custom_fields") or [])
    _write_validation_rules(pack_dir, metadata.get("validation_rules") or [])
    _write_permission_sets(pack_dir, metadata.get("permission_sets") or [])
    return pack_dir


def _generate_hubspot_pack(proposal: dict[str, Any], project_dir: Path) -> Path:
    pack_dir = project_dir / "proposal" / "hubspot_pack"
    _ensure_safe_pack_dir(pack_dir, project_dir)
    if pack_dir.exists():
        shutil.rmtree(pack_dir)
    hubspot_pack.init_pack(pack_dir)

    pack = proposal.get("hubspot_pack") or {}
    properties = pack.get("properties") or []
    property_groups = pack.get("property_groups") or []
    pipelines = pack.get("pipelines") or []
    workflow_templates = pack.get("workflow_templates") or []

    objects: dict[str, dict[str, Any]] = {}
    for group in property_groups:
        if not isinstance(group, dict):
            continue
        obj = group.get("object") or group.get("object_type") or "contacts"
        entry = objects.setdefault(str(obj).lower(), {"groups": [], "properties": []})
        entry["groups"].append(group)
    for prop in properties:
        if not isinstance(prop, dict):
            continue
        obj = prop.get("object") or prop.get("object_type") or "contacts"
        entry = objects.setdefault(str(obj).lower(), {"groups": [], "properties": []})
        entry["properties"].append(prop)

    if objects:
        properties_path = pack_dir / "properties.yaml"
        properties_payload = {"version": 1, "objects": objects}
        properties_path.write_text(json.dumps(properties_payload, indent=2, ensure_ascii=True), encoding="utf-8")

    pipelines_dir = pack_dir / "pipelines"
    pipelines_dir.mkdir(parents=True, exist_ok=True)
    for idx, pipeline in enumerate(pipelines, start=1):
        if not isinstance(pipeline, dict):
            continue
        name = pipeline.get("name") or pipeline.get("pipeline_id") or pipeline.get("id") or f"pipeline_{idx}"
        safe_name = _safe_api_name(str(name), append_custom_suffix=False)
        (pipelines_dir / f"{safe_name}.json").write_text(
            json.dumps(pipeline, indent=2, ensure_ascii=True),
            encoding="utf-8",
        )

    workflows_dir = pack_dir / "workflows"
    workflows_dir.mkdir(parents=True, exist_ok=True)
    for idx, flow in enumerate(workflow_templates, start=1):
        if not isinstance(flow, dict):
            continue
        name = flow.get("name") or flow.get("id") or flow.get("flowId") or f"workflow_{idx}"
        safe_name = _safe_api_name(str(name), append_custom_suffix=False)
        (workflows_dir / f"{safe_name}.json").write_text(
            json.dumps(flow, indent=2, ensure_ascii=True),
            encoding="utf-8",
        )

    return pack_dir


def generate_proposal_markdown(proposal: dict[str, Any]) -> str:
    summary = proposal.get("summary") or {}
    metadata = proposal.get("metadata_pack") or {}
    hubspot_pack_data = proposal.get("hubspot_pack") or {}
    lines = [
        f"# {summary.get('title', 'Architect Proposal')}",
        "",
        "## Goals",
    ]
    for goal in summary.get("goals") or []:
        lines.append(f"- {goal}")
    lines.append("")
    lines.append("## Risks")
    for risk in summary.get("risks") or []:
        lines.append(f"- {risk}")
    if summary.get("notes"):
        lines.append("")
        lines.append("## Notes")
        lines.append(str(summary.get("notes")))

    lines.append("")
    if metadata:
        lines.append("## Metadata Pack")
        lines.append(f"- Custom fields: {len(metadata.get('custom_fields') or [])}")
        lines.append(f"- Validation rules: {len(metadata.get('validation_rules') or [])}")
        lines.append(f"- Permission sets: {len(metadata.get('permission_sets') or [])}")
    if hubspot_pack_data:
        lines.append("## HubSpot Pack")
        lines.append(f"- Properties: {len(hubspot_pack_data.get('properties') or [])}")
        lines.append(f"- Pipelines: {len(hubspot_pack_data.get('pipelines') or [])}")
        lines.append(f"- Workflows: {len(hubspot_pack_data.get('workflow_templates') or [])}")

    matching = proposal.get("matching_config") or {}
    if matching:
        lines.append("")
        lines.append("## Matching Config")
        for key, value in matching.items():
            lines.append(f"- {key}: {value}")

    return "\n".join(lines)


def sanitize_artifact_text(text: str) -> str:
    if not text:
        return text
    redacted = EMAIL_PATTERN.sub("<redacted-email>", text)
    redacted = SFID_PATTERN.sub("<redacted-id>", redacted)
    redacted = USERNAME_PATTERN.sub(r"\1<redacted-user>", redacted)
    return redacted


def sanitize_artifact_json(payload: Any) -> Any:
    if isinstance(payload, dict):
        sanitized = {}
        for key, value in payload.items():
            lowered = key.lower()
            if lowered in {"username", "user", "user_id", "owner_id"} and isinstance(value, str):
                sanitized[key] = "<redacted-user>"
                continue
            sanitized[key] = sanitize_artifact_json(value)
        return sanitized
    if isinstance(payload, list):
        return [sanitize_artifact_json(item) for item in payload]
    if isinstance(payload, str):
        return sanitize_artifact_text(payload)
    return payload


def _ensure_safe_pack_dir(pack_dir: Path, project_dir: Path) -> None:
    resolved_pack = pack_dir.resolve()
    resolved_project = project_dir.resolve()
    try:
        resolved_pack.relative_to(resolved_project)
    except Exception as exc:
        raise ValueError(f"Metadata pack dir must be within project_dir: {resolved_pack}") from exc


def _hash_payload(payload: dict[str, Any]) -> str:
    raw = json.dumps(payload, sort_keys=True, separators=(",", ":"), ensure_ascii=True)
    return f"sha256:{hashlib.sha256(raw.encode('utf-8')).hexdigest()}"


def _write_custom_fields(pack_dir: Path, fields: list[dict[str, Any]]) -> None:
    root = pack_dir / "force-app" / "main" / "default" / "objects"
    for field in fields:
        obj = field.get("object") or "Lead"
        api_name = _safe_api_name(
            field.get("api_name") or field.get("label") or "Custom_Field",
            append_custom_suffix=True,
        )
        path = root / obj / "fields" / f"{api_name}.field-meta.xml"
        path.parent.mkdir(parents=True, exist_ok=True)
        label = field.get("label") or api_name
        field_type = field.get("type") or "Text"
        length = field.get("length")
        required = "true" if field.get("required") else "false"
        description = field.get("description") or ""
        xml = [
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
            "<CustomField xmlns=\"http://soap.sforce.com/2006/04/metadata\">",
            f"  <fullName>{api_name}</fullName>",
            f"  <label>{label}</label>",
            f"  <type>{field_type}</type>",
            f"  <required>{required}</required>",
        ]
        if length and str(length).isdigit():
            xml.append(f"  <length>{int(length)}</length>")
        if description:
            xml.append(f"  <description>{description}</description>")
        xml.append("</CustomField>")
        path.write_text("\n".join(xml), encoding="utf-8")


def _write_validation_rules(pack_dir: Path, rules: list[dict[str, Any]]) -> None:
    root = pack_dir / "force-app" / "main" / "default" / "objects"
    for rule in rules:
        obj = rule.get("object") or "Lead"
        api_name = _safe_api_name(
            rule.get("api_name") or rule.get("label") or "Validation_Rule",
            append_custom_suffix=False,
        )
        path = root / obj / "validationRules" / f"{api_name}.validationRule-meta.xml"
        path.parent.mkdir(parents=True, exist_ok=True)
        active = "true" if rule.get("active", True) else "false"
        condition = rule.get("error_condition") or "FALSE"
        message = rule.get("error_message") or "Validation failed."
        description = rule.get("description") or ""
        xml = [
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
            "<ValidationRule xmlns=\"http://soap.sforce.com/2006/04/metadata\">",
            f"  <fullName>{api_name}</fullName>",
            f"  <active>{active}</active>",
            f"  <errorConditionFormula>{condition}</errorConditionFormula>",
            f"  <errorMessage>{message}</errorMessage>",
        ]
        if description:
            xml.append(f"  <description>{description}</description>")
        xml.append("</ValidationRule>")
        path.write_text("\n".join(xml), encoding="utf-8")


def _write_permission_sets(pack_dir: Path, permsets: list[dict[str, Any]]) -> None:
    root = pack_dir / "force-app" / "main" / "default" / "permissionsets"
    for perm in permsets:
        api_name = _safe_api_name(
            perm.get("api_name") or perm.get("label") or "L2A_Permission",
            append_custom_suffix=False,
        )
        path = root / f"{api_name}.permissionset-meta.xml"
        label = perm.get("label") or api_name
        xml = [
            "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
            "<PermissionSet xmlns=\"http://soap.sforce.com/2006/04/metadata\">",
            f"  <label>{label}</label>",
        ]
        for fp in perm.get("field_permissions") or []:
            obj = fp.get("object") or "Lead"
            field_name = fp.get("field") or ""
            if not field_name:
                continue
            readable = "true" if fp.get("readable", True) else "false"
            editable = "true" if fp.get("editable", False) else "false"
            xml.extend(
                [
                    "  <fieldPermissions>",
                    f"    <field>{obj}.{field_name}</field>",
                    f"    <readable>{readable}</readable>",
                    f"    <editable>{editable}</editable>",
                    "  </fieldPermissions>",
                ]
            )
        xml.append("</PermissionSet>")
        path.write_text("\n".join(xml), encoding="utf-8")


def _safe_api_name(value: str, *, append_custom_suffix: bool) -> str:
    text = re.sub(r"[^A-Za-z0-9_]", "_", str(value)).strip("_")
    if not text:
        text = "Custom_Field"
    if append_custom_suffix and not text.endswith("__c"):
        text = f"{text}__c"
    return text


__all__ = [
    "generate_proposal",
    "generate_skeleton",
    "generate_metadata_pack",
    "generate_proposal_markdown",
    "sanitize_artifact_text",
    "sanitize_artifact_json",
]



"""
Email Notification Service for scheduled jobs and auth failures
"""

import logging
from typing import Optional
import os

log = logging.getLogger(__name__)

PROACTIVE_OUTREACH_SUBJECT = "Quick check-in from FoundryOps"
PROACTIVE_OUTREACH_TEMPLATE = """{greeting}

I noticed the FoundryOps add-on may have run into an issue while
you were working earlier today.

If you're blocked or need help, just reply to this email or hit
the SOS button in the add-on — I'd be happy to take a look.

If everything's working fine, no action needed!

— Mike
"""


async def send_proactive_outreach_email(
    user_email: str, first_name: Optional[str] = None
) -> Optional[str]:
    """
    Send a non-intrusive proactive outreach email.

    Returns:
        Resend email ID when available, else None.
    """
    greeting = f"Hi {first_name}," if first_name else "Hi,"
    body = PROACTIVE_OUTREACH_TEMPLATE.format(greeting=greeting)

    email_provider = os.getenv("EMAIL_PROVIDER", "").lower()
    if email_provider == "resend":
        return await _send_via_resend_with_id(
            user_email,
            PROACTIVE_OUTREACH_SUBJECT,
            body,
            tags=[{"name": "message_type", "value": "support_proactive"}],
        )

    await _send_email(user_email, PROACTIVE_OUTREACH_SUBJECT, body)
    return None


async def send_auth_failure_notification(
    user_email: str,
    integration_provider: str,
    account_id: str,
    failure_reason: Optional[str] = None,
) -> bool:
    """
    Send email notification when integration credentials expire or fail.

    Args:
        user_email: User's email address
        integration_provider: Name of integration (e.g., "Salesforce")
        account_id: Account ID for logging
        failure_reason: Optional reason for failure

    Returns:
        True if email sent successfully, False otherwise
    """
    try:
        subject = f"{integration_provider} Connection Expired - Action Required"

        provider = (integration_provider or "").strip().lower()
        if provider.startswith("salesforce"):
            steps = [
                "1. Open your Google Sheet",
                "2. Go to Extensions -> FoundryMatch -> Salesforce -> Connection Status",
                '3. Click "Reconnect" to re-authorize',
            ]
        else:
            steps = [
                "1. Open your Google Sheet",
                "2. Go to Extensions -> FoundryMatch -> Schedule Manager",
                '3. Click "Connect Google" to re-authorize',
            ]

        body_lines = [
            "Hello,",
            "",
            f"Your {integration_provider} connection has expired and scheduled automations have been paused.",
            "",
            f"Reason: {failure_reason}" if failure_reason else "",
            "",
            "To resume your scheduled reports and data syncs:",
            "",
            *steps,
            "",
            "Your schedules will automatically resume once reconnected.",
            "",
            "---",
            "FoundryMatch Team",
            "https://foundrymatch.com",
            "",
            "(This is an automated notification)",
        ]
        body = "\n".join(line for line in body_lines if line)

        # Send email via configured provider
        await _send_email(user_email, subject, body)

        log.info(
            f"Sent auth failure notification to {user_email} for {integration_provider}"
        )
        return True

    except Exception as e:
        log.error(f"Failed to send auth failure notification: {e}")
        return False


async def send_schedule_paused_notification(
    user_email: str,
    schedule_name: str,
    job_id: str,
    error_message: str,
    account_id: str,
) -> bool:
    """
    Send email when a scheduled job is paused due to errors.
    """
    try:
        subject = f"Scheduled Job Paused: {schedule_name}"

        body_lines = [
            "Hello,",
            "",
            f'Your scheduled job "{schedule_name}" has been automatically paused after multiple failures.',
            "",
            f"Error: {error_message}",
            "",
            "To fix this:",
            "",
            "1. Open your Google Sheet",
            "2. Go to Extensions -> FoundryMatch -> Schedule Manager",
            "3. Review the job settings and error details",
            "4. Fix any issues (expired credentials, invalid queries, etc.)",
            '5. Click "Resume" to restart the schedule',
            "",
            f"Job ID: {job_id}",
            "",
            "---",
            "FoundryMatch Team",
            "https://foundrymatch.com",
        ]
        body = "\n".join(line for line in body_lines if line)

        await _send_email(user_email, subject, body)

        log.info(f"Sent schedule paused notification to {user_email} for job {job_id}")
        return True

    except Exception as e:
        log.error(f"Failed to send schedule paused notification: {e}")
        return False


async def send_sheet_webhook_failure_notification(
    user_email: str,
    webhook_name: str,
    sheet_name: str,
    error_message: str,
    account_id: str,
) -> bool:
    """
    Send email when a sheet webhook is failing after ingest accepted the request.
    """
    try:
        subject = f"Webhook Needs Attention: {webhook_name}"

        body_lines = [
            "Hello,",
            "",
            f'Your sheet webhook "{webhook_name}" is currently failing to write into "{sheet_name}".',
            "",
            f"Latest error: {error_message}",
            "",
            "To fix this:",
            "",
            "1. Open your Google Sheet",
            "2. Go to Extensions -> FoundryMatch -> Webhook Manager",
            "3. Review the failing webhook and reconnect Google or fix the target sheet",
            '4. Use the "History" panel to replay failed deliveries after the issue is fixed',
            "",
            f"Account ID: {account_id}",
            "",
            "---",
            "FoundryMatch Team",
            "https://foundryops.io",
        ]
        body = "\n".join(line for line in body_lines if line)

        await _send_email(user_email, subject, body)

        log.info(
            "Sent sheet webhook failure notification to %s for webhook %s",
            user_email,
            webhook_name,
        )
        return True

    except Exception as e:
        log.error(f"Failed to send sheet webhook failure notification: {e}")
        return False


async def send_enrichment_complete_notification(
    user_email: str,
    job_name: str,
    job_id: str,
    total_contacts: int,
    enriched_count: int,
    not_found_count: int,
    error_count: int,
    spreadsheet_url: Optional[str] = None,
    record_label: str = "contacts",
) -> bool:
    """
    Send branded HTML email notification when enrichment completes.
    """
    try:
        success_rate = (enriched_count / total_contacts * 100) if total_contacts > 0 else 0
        label = (record_label or "contacts").strip() or "contacts"
        label_title = label.capitalize()
        subject = f"✅ Enrichment Complete: {enriched_count}/{total_contacts} {label} enriched"
        error_color = "#ef4444" if error_count > 0 else "#6b7280"

        cta_html = ""
        if spreadsheet_url:
            cta_html = f"""
              <table width="100%" cellpadding="0" cellspacing="0">
                <tr>
                  <td align="center">
                    <a href="{spreadsheet_url}" style="display: inline-block; background: linear-gradient(135deg, #f97316 0%, #ea580c 100%); color: #ffffff; text-decoration: none; padding: 14px 32px; border-radius: 8px; font-weight: 600; font-size: 15px;">
                      View Results in Google Sheets →
                    </a>
                  </td>
                </tr>
              </table>
            """

        text_body_lines = [
            "Hello,",
            "",
            f"Your {job_name} job has finished processing.",
            "",
            f"Total Rows: {total_contacts}",
            f"Enriched: {enriched_count}",
            f"Not Found: {not_found_count}",
            f"Errors: {error_count}",
            "",
            f"Job ID: {job_id}",
            "",
            "FoundryMatch Team",
        ]
        text_body = "\n".join(text_body_lines)

        html_body = f"""
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body style="margin: 0; padding: 0; background-color: #f4f4f5; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;">
  <table width="100%" cellpadding="0" cellspacing="0" style="background-color: #f4f4f5; padding: 40px 20px;">
    <tr>
      <td align="center">
        <table width="600" cellpadding="0" cellspacing="0" style="background-color: #ffffff; border-radius: 12px; overflow: hidden; box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);">
          
          <tr>
            <td style="background: linear-gradient(135deg, #0a0a0a 0%, #1a1a1a 100%); padding: 32px 40px; text-align: center;">
              <div style="width: 48px; height: 48px; line-height: 48px; border-radius: 12px; background: #111827; color: #f97316; font-weight: 700; font-size: 18px; display: inline-block; text-align: center; margin-bottom: 12px;">FM</div>
              <h1 style="color: #f97316; font-size: 24px; font-weight: 700; margin: 0;">FoundryMatch</h1>
            </td>
          </tr>
          
          <tr>
            <td style="background: linear-gradient(135deg, #22c55e 0%, #16a34a 100%); padding: 24px 40px; text-align: center;">
              <span style="font-size: 32px;">✅</span>
              <h2 style="color: #ffffff; font-size: 20px; font-weight: 600; margin: 8px 0 0 0;">Enrichment Complete</h2>
            </td>
          </tr>
          
          <tr>
            <td style="padding: 40px;">
              <p style="color: #374151; font-size: 16px; line-height: 1.6; margin: 0 0 24px 0;">
                Your {job_name} job has finished processing. Here's a summary of the results:
              </p>
              
              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 24px;">
                <tr>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 8px 0 0 8px; text-align: center; border-right: 1px solid #e5e7eb;">
                    <div style="font-size: 28px; font-weight: 700; color: #22c55e;">{enriched_count:,}</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Enriched</div>
                  </td>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 0 8px 8px 0; text-align: center;">
                    <div style="font-size: 28px; font-weight: 700; color: #f97316;">{success_rate:.0f}%</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Success Rate</div>
                  </td>
                </tr>
              </table>
              
              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 32px;">
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Total {label_title}</span>
                    <span style="float: right; font-weight: 600; color: #111827;">{total_contacts:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Successfully Enriched</span>
                    <span style="float: right; font-weight: 600; color: #22c55e;">{enriched_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Not Found</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{not_found_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0;">
                    <span style="color: #374151;">Errors</span>
                    <span style="float: right; font-weight: 600; color: {error_color};">{error_count:,}</span>
                  </td>
                </tr>
              </table>
              
              {cta_html}
              
            </td>
          </tr>
          
          <tr>
            <td style="background: #f9fafb; padding: 24px 40px; text-align: center; border-top: 1px solid #e5e7eb;">
              <p style="color: #6b7280; font-size: 13px; margin: 0 0 8px 0;">
                Job ID: {job_id}
              </p>
              <p style="color: #9ca3af; font-size: 12px; margin: 0;">
                <a href="https://foundryops.io" style="color: #f97316; text-decoration: none;">FoundryMatch</a> by FoundryOps
              </p>
            </td>
          </tr>
          
        </table>
      </td>
    </tr>
  </table>
</body>
</html>
"""

        text_body = f"""
Your {job_name} job has completed.

Results Summary:
? Total {label}: {total_contacts:,}
? Enriched: {enriched_count:,} ({success_rate:.1f}%)
? Not found: {not_found_count:,}
? Errors: {error_count:,}

{f"View your results: {spreadsheet_url}" if spreadsheet_url else ""}

Job ID: {job_id}

---
FoundryMatch by FoundryOps
https://foundryops.io
"""

        await _send_email(user_email, subject, text_body, html_body=html_body)

        log.info(
            f"Sent enrichment complete notification to {user_email} for job {job_id}"
        )
        return True

    except Exception as e:
        log.error(f"Failed to send enrichment complete notification: {e}")
        return False


async def send_scraper_complete_notification(
    user_email: str,
    job_name: str,
    job_id: str,
    total_rows: int,
    scraped_count: int,
    skipped_count: int,
    error_count: int,
    spreadsheet_url: Optional[str] = None,
) -> bool:
    """
    Send branded HTML email notification when a scraper job completes.
    """
    try:
        success_rate = (scraped_count / total_rows * 100) if total_rows > 0 else 0
        subject = f"✅ Scraper Complete: {scraped_count}/{total_rows} rows scraped"
        error_color = "#ef4444" if error_count > 0 else "#6b7280"

        cta_html = ""
        if spreadsheet_url:
            cta_html = f"""
              <table width="100%" cellpadding="0" cellspacing="0">
                <tr>
                  <td align="center">
                    <a href="{spreadsheet_url}" style="display: inline-block; background: linear-gradient(135deg, #f97316 0%, #ea580c 100%); color: #ffffff; text-decoration: none; padding: 14px 32px; border-radius: 8px; font-weight: 600; font-size: 15px;">
                      View Results in Google Sheets →
                    </a>
                  </td>
                </tr>
              </table>
            """

        html_body = f"""
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body style="margin: 0; padding: 0; background-color: #f4f4f5; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;">
  <table width="100%" cellpadding="0" cellspacing="0" style="background-color: #f4f4f5; padding: 40px 20px;">
    <tr>
      <td align="center">
        <table width="600" cellpadding="0" cellspacing="0" style="background-color: #ffffff; border-radius: 12px; overflow: hidden; box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);">
          
          <tr>
            <td style="background: linear-gradient(135deg, #0a0a0a 0%, #1a1a1a 100%); padding: 32px 40px; text-align: center;">
              <div style="width: 48px; height: 48px; line-height: 48px; border-radius: 12px; background: #111827; color: #f97316; font-weight: 700; font-size: 18px; display: inline-block; text-align: center; margin-bottom: 12px;">FM</div>
              <h1 style="color: #f97316; font-size: 24px; font-weight: 700; margin: 0;">FoundryMatch</h1>
            </td>
          </tr>
          
          <tr>
            <td style="background: linear-gradient(135deg, #22c55e 0%, #16a34a 100%); padding: 24px 40px; text-align: center;">
              <span style="font-size: 32px;">✅</span>
              <h2 style="color: #ffffff; font-size: 20px; font-weight: 600; margin: 8px 0 0 0;">Scraper Complete</h2>
            </td>
          </tr>
          
          <tr>
            <td style="padding: 40px;">
              <p style="color: #374151; font-size: 16px; line-height: 1.6; margin: 0 0 24px 0;">
                Your {job_name} job has finished processing. Here's a summary of the results:
              </p>
              
              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 24px;">
                <tr>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 8px 0 0 8px; text-align: center; border-right: 1px solid #e5e7eb;">
                    <div style="font-size: 28px; font-weight: 700; color: #22c55e;">{scraped_count:,}</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Scraped</div>
                  </td>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 0 8px 8px 0; text-align: center;">
                    <div style="font-size: 28px; font-weight: 700; color: #f97316;">{success_rate:.0f}%</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Success Rate</div>
                  </td>
                </tr>
              </table>
              
              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 32px;">
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Total Rows</span>
                    <span style="float: right; font-weight: 600; color: #111827;">{total_rows:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Scraped</span>
                    <span style="float: right; font-weight: 600; color: #22c55e;">{scraped_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Skipped</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{skipped_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0;">
                    <span style="color: #374151;">Errors</span>
                    <span style="float: right; font-weight: 600; color: {error_color};">{error_count:,}</span>
                  </td>
                </tr>
              </table>

              {cta_html}
              
              <p style="color: #9ca3af; font-size: 12px; margin: 32px 0 0 0; text-align: center;">
                Job ID: {job_id}
              </p>
            </td>
          </tr>
          
          <tr>
            <td style="background: #f9fafb; padding: 24px 40px; text-align: center; border-top: 1px solid #e5e7eb;">
              <p style="color: #6b7280; font-size: 12px; margin: 0;">
                FoundryMatch by FoundryOps • <a href="https://foundryops.io" style="color: #f97316; text-decoration: none;">foundryops.io</a>
              </p>
            </td>
          </tr>
          
        </table>
      </td>
    </tr>
  </table>
</body>
</html>
"""

        text_body = f"""
Your {job_name} job has completed.

Results Summary:
- Total rows: {total_rows:,}
- Scraped: {scraped_count:,} ({success_rate:.1f}%)
- Skipped: {skipped_count:,}
- Errors: {error_count:,}

{f"View your results: {spreadsheet_url}" if spreadsheet_url else ""}

Job ID: {job_id}

---
FoundryMatch by FoundryOps
https://foundryops.io
"""

        await _send_email(user_email, subject, text_body, html_body=html_body)

        log.info(
            f"Sent scraper complete notification to {user_email} for job {job_id}"
        )
        return True

    except Exception as e:
        log.error(f"Failed to send scraper complete notification: {e}")
        return False


async def send_justcall_complete_notification(
    user_email: str,
    job_name: str,
    job_id: str,
    total_rows: int,
    enrolled_count: int,
    failed_count: int,
    skipped_count: int,
    spreadsheet_url: Optional[str] = None,
) -> bool:
    """
    Send branded HTML email notification when a JustCall autodialer push completes.
    """
    try:
        success_rate = (enrolled_count / total_rows * 100) if total_rows > 0 else 0
        subject = f"Autodialer Push Complete: {enrolled_count}/{total_rows} contacts enrolled"
        error_color = "#ef4444" if failed_count > 0 else "#6b7280"

        cta_html = ""
        if spreadsheet_url:
            cta_html = f"""
              <table width="100%" cellpadding="0" cellspacing="0">
                <tr>
                  <td align="center">
                    <a href="{spreadsheet_url}" style="display: inline-block; background: linear-gradient(135deg, #f97316 0%, #ea580c 100%); color: #ffffff; text-decoration: none; padding: 14px 32px; border-radius: 8px; font-weight: 600; font-size: 15px;">
                      View Results in Google Sheets
                    </a>
                  </td>
                </tr>
              </table>
            """

        text_body = "\n".join(
            [
                "Hello,",
                "",
                f"Your {job_name} job has finished processing.",
                "",
                f"Total Rows: {total_rows}",
                f"Enrolled: {enrolled_count}",
                f"Skipped: {skipped_count}",
                f"Failed: {failed_count}",
                "",
                f"Job ID: {job_id}",
                "",
                "FoundryMatch Team",
            ]
        )

        html_body = f"""
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body style="margin: 0; padding: 0; background-color: #f4f4f5; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;">
  <table width="100%" cellpadding="0" cellspacing="0" style="background-color: #f4f4f5; padding: 40px 20px;">
    <tr>
      <td align="center">
        <table width="600" cellpadding="0" cellspacing="0" style="background-color: #ffffff; border-radius: 12px; overflow: hidden; box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);">

          <tr>
            <td style="background: linear-gradient(135deg, #0a0a0a 0%, #1a1a1a 100%); padding: 32px 40px; text-align: center;">
              <div style="width: 48px; height: 48px; line-height: 48px; border-radius: 12px; background: #111827; color: #f97316; font-weight: 700; font-size: 18px; display: inline-block; text-align: center; margin-bottom: 12px;">FM</div>
              <h1 style="color: #f97316; font-size: 24px; font-weight: 700; margin: 0;">FoundryMatch</h1>
            </td>
          </tr>

          <tr>
            <td style="background: linear-gradient(135deg, #2563eb 0%, #1d4ed8 100%); padding: 24px 40px; text-align: center;">
              <span style="font-size: 32px;">OK</span>
              <h2 style="color: #ffffff; font-size: 20px; font-weight: 600; margin: 8px 0 0 0;">Autodialer Push Complete</h2>
            </td>
          </tr>

          <tr>
            <td style="padding: 40px;">
              <p style="color: #374151; font-size: 16px; line-height: 1.6; margin: 0 0 24px 0;">
                Your {job_name} job has finished processing. Here's a summary of the results:
              </p>

              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 24px;">
                <tr>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 8px 0 0 8px; text-align: center; border-right: 1px solid #e5e7eb;">
                    <div style="font-size: 28px; font-weight: 700; color: #22c55e;">{enrolled_count:,}</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Enrolled</div>
                  </td>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 0 8px 8px 0; text-align: center;">
                    <div style="font-size: 28px; font-weight: 700; color: #f97316;">{success_rate:.0f}%</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Success Rate</div>
                  </td>
                </tr>
              </table>

              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 32px;">
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Total Rows</span>
                    <span style="float: right; font-weight: 600; color: #111827;">{total_rows:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Enrolled</span>
                    <span style="float: right; font-weight: 600; color: #22c55e;">{enrolled_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Skipped</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{skipped_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0;">
                    <span style="color: #374151;">Failed</span>
                    <span style="float: right; font-weight: 600; color: {error_color};">{failed_count:,}</span>
                  </td>
                </tr>
              </table>

              {cta_html}
            </td>
          </tr>

          <tr>
            <td style="background: #f9fafb; padding: 20px 40px; text-align: center; color: #9ca3af; font-size: 12px;">
              Job ID: {job_id}
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
</body>
</html>
"""

        await _send_email(user_email, subject, text_body, html_body=html_body)

        log.info(
            f"Sent JustCall completion notification to {user_email} for job {job_id}"
        )
        return True

    except Exception as e:
        log.error(f"Failed to send JustCall completion notification: {e}")
        return False


async def send_heyreach_complete_notification(
    user_email: str,
    job_name: str,
    job_id: str,
    total_rows: int,
    enrolled_count: int,
    failed_count: int,
    skipped_count: int,
    duplicate_count: int = 0,
    invalid_count: int = 0,
    spreadsheet_url: Optional[str] = None,
) -> bool:
    """
    Send branded HTML email notification when a HeyReach LinkedIn push completes.
    """
    try:
        success_rate = (enrolled_count / total_rows * 100) if total_rows > 0 else 0
        subject = f"LinkedIn Push Complete: {enrolled_count}/{total_rows} contacts enrolled"
        error_color = "#ef4444" if failed_count > 0 else "#6b7280"

        cta_html = ""
        if spreadsheet_url:
            cta_html = f"""
              <table width="100%" cellpadding="0" cellspacing="0">
                <tr>
                  <td align="center">
                    <a href="{spreadsheet_url}" style="display: inline-block; background: linear-gradient(135deg, #f97316 0%, #ea580c 100%); color: #ffffff; text-decoration: none; padding: 14px 32px; border-radius: 8px; font-weight: 600; font-size: 15px;">
                      View Results in Google Sheets
                    </a>
                  </td>
                </tr>
              </table>
            """

        html_body = f"""
<!DOCTYPE html>
<html>
<head>
  <meta charset="utf-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0">
</head>
<body style="margin: 0; padding: 0; background-color: #f4f4f5; font-family: -apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial, sans-serif;">
  <table width="100%" cellpadding="0" cellspacing="0" style="background-color: #f4f4f5; padding: 40px 20px;">
    <tr>
      <td align="center">
        <table width="600" cellpadding="0" cellspacing="0" style="background-color: #ffffff; border-radius: 12px; overflow: hidden; box-shadow: 0 4px 6px rgba(0, 0, 0, 0.1);">

          <tr>
            <td style="background: linear-gradient(135deg, #0a0a0a 0%, #1a1a1a 100%); padding: 32px 40px; text-align: center;">
              <div style="width: 48px; height: 48px; line-height: 48px; border-radius: 12px; background: #111827; color: #f97316; font-weight: 700; font-size: 18px; display: inline-block; text-align: center; margin-bottom: 12px;">FM</div>
              <h1 style="color: #f97316; font-size: 24px; font-weight: 700; margin: 0;">FoundryMatch</h1>
            </td>
          </tr>

          <tr>
            <td style="background: linear-gradient(135deg, #0077b5 0%, #005582 100%); padding: 24px 40px; text-align: center;">
              <span style="font-size: 32px;">in</span>
              <h2 style="color: #ffffff; font-size: 20px; font-weight: 600; margin: 8px 0 0 0;">LinkedIn Push Complete</h2>
            </td>
          </tr>

          <tr>
            <td style="padding: 40px;">
              <p style="color: #374151; font-size: 16px; line-height: 1.6; margin: 0 0 24px 0;">
                Your {job_name} job has finished processing. Here's a summary of the results:
              </p>

              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 24px;">
                <tr>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 8px 0 0 8px; text-align: center; border-right: 1px solid #e5e7eb;">
                    <div style="font-size: 28px; font-weight: 700; color: #22c55e;">{enrolled_count:,}</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Enrolled</div>
                  </td>
                  <td width="50%" style="padding: 16px; background: #f9fafb; border-radius: 0 8px 8px 0; text-align: center;">
                    <div style="font-size: 28px; font-weight: 700; color: #f97316;">{success_rate:.0f}%</div>
                    <div style="font-size: 13px; color: #6b7280; margin-top: 4px;">Success Rate</div>
                  </td>
                </tr>
              </table>

              <table width="100%" cellpadding="0" cellspacing="0" style="margin-bottom: 32px;">
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Total Rows</span>
                    <span style="float: right; font-weight: 600; color: #111827;">{total_rows:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Enrolled</span>
                    <span style="float: right; font-weight: 600; color: #22c55e;">{enrolled_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Skipped</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{skipped_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Duplicate</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{duplicate_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0; border-bottom: 1px solid #e5e7eb;">
                    <span style="color: #374151;">Invalid</span>
                    <span style="float: right; font-weight: 600; color: #6b7280;">{invalid_count:,}</span>
                  </td>
                </tr>
                <tr>
                  <td style="padding: 12px 0;">
                    <span style="color: #374151;">Failed</span>
                    <span style="float: right; font-weight: 600; color: {error_color};">{failed_count:,}</span>
                  </td>
                </tr>
              </table>

              {cta_html}
            </td>
          </tr>

          <tr>
            <td style="background: #f9fafb; padding: 20px 40px; text-align: center; color: #9ca3af; font-size: 12px;">
              Job ID: {job_id}
            </td>
          </tr>
        </table>
      </td>
    </tr>
  </table>
</body>
</html>
"""

        text_body = f"""
Your {job_name} job has completed.

Results Summary:
- Total rows: {total_rows:,}
- Enrolled: {enrolled_count:,} ({success_rate:.1f}%)
- Skipped: {skipped_count:,}
- Duplicate: {duplicate_count:,}
- Invalid: {invalid_count:,}
- Failed: {failed_count:,}

{f"View your results: {spreadsheet_url}" if spreadsheet_url else ""}

Job ID: {job_id}

---
FoundryMatch by FoundryOps
https://foundryops.io
"""

        await _send_email(user_email, subject, text_body, html_body=html_body)

        log.info(
            f"Sent HeyReach completion notification to {user_email} for job {job_id}"
        )
        return True

    except Exception as e:
        log.error(f"Failed to send HeyReach completion notification: {e}")
        return False


async def _send_email(
    to: str,
    subject: str,
    body: str,
    html_body: Optional[str] = None,
) -> None:
    """
    Send email via configured provider (Resend, SendGrid, SES, SMTP)
    """
    # Check which email provider is configured
    email_provider = os.getenv("EMAIL_PROVIDER", "").lower()

    if email_provider == "resend":
        await _send_via_resend(to, subject, body, html_body=html_body)
    elif email_provider == "sendgrid":
        await _send_via_sendgrid(to, subject, body)
    elif email_provider == "ses":
        await _send_via_ses(to, subject, body)
    elif email_provider == "smtp":
        await _send_via_smtp(to, subject, body)
    else:
        # Log to console for now (dev mode)
        log.warning(
            f"Email provider not configured. Email content:\nTo: {to}\nSubject: {subject}\n{body}"
        )


async def _send_via_resend_with_id(
    to: str,
    subject: str,
    body: str,
    html_body: Optional[str] = None,
    tags: Optional[list[dict]] = None,
) -> Optional[str]:
    """Send email via Resend and return the email ID when available."""
    try:
        import httpx

        api_key = os.getenv("RESEND_API_KEY")
        from_email = os.getenv("RESEND_FROM_EMAIL", "notifications@foundryops.io")

        if not api_key:
            log.warning("Resend API key not configured (RESEND_API_KEY)")
            return None

        email_payload: dict[str, object] = {
            "from": f"FoundryOps <{from_email}>",
            "to": [to],
            "subject": subject,
            "text": body,
        }
        if html_body:
            email_payload["html"] = html_body
        if tags:
            email_payload["tags"] = tags

        async with httpx.AsyncClient() as client:
            response = await client.post(
                "https://api.resend.com/emails",
                headers={
                    "Authorization": f"Bearer {api_key}",
                    "Content-Type": "application/json",
                },
                json=email_payload,
                timeout=20,
            )
            if response.status_code >= 300:
                log.warning(
                    "Resend send failed (%s): %s",
                    response.status_code,
                    response.text,
                )
                return None
            data = response.json()
            return data.get("id") if isinstance(data, dict) else None
    except Exception as exc:
        log.error("Resend send failed: %s", exc)
        return None


async def _send_via_resend(
    to: str,
    subject: str,
    body: str,
    html_body: Optional[str] = None,
) -> None:
    """Send email via Resend (https://resend.com)"""
    try:
        import httpx

        api_key = os.getenv("RESEND_API_KEY")
        from_email = os.getenv("RESEND_FROM_EMAIL", "notifications@foundryops.io")

        if not api_key:
            log.warning("Resend API key not configured (RESEND_API_KEY)")
            return

        email_payload = {
            "from": f"FoundryOps <{from_email}>",
            "to": [to],
            "subject": subject,
            "text": body,
        }
        if html_body:
            email_payload["html"] = html_body

        async with httpx.AsyncClient() as client:
            response = await client.post(
                "https://api.resend.com/emails",
                headers={
                    "Authorization": f"Bearer {api_key}",
                    "Content-Type": "application/json",
                },
                json=email_payload,
                timeout=10.0,
            )

            if response.status_code not in (200, 201):
                log.error(f"Resend error: {response.status_code} - {response.text}")
                raise Exception(f"Resend failed: {response.status_code}")

            result = response.json()
            log.info(f"Resend email sent: {result.get('id')}")

    except Exception as e:
        log.error(f"Resend send failed: {e}")
        raise


async def _send_via_sendgrid(to: str, subject: str, body: str) -> None:
    """Send email via SendGrid"""
    try:
        import httpx
        from ... import settings

        api_key = os.getenv("SENDGRID_API_KEY") or settings.SENDGRID_API_KEY
        from_email = os.getenv("SENDGRID_FROM_EMAIL", "notifications@foundrymatch.com")

        if not api_key:
            log.warning("SendGrid API key not configured")
            return

        async with httpx.AsyncClient() as client:
            response = await client.post(
                "https://api.sendgrid.com/v3/mail/send",
                headers={
                    "Authorization": f"Bearer {api_key}",
                    "Content-Type": "application/json",
                },
                json={
                    "personalizations": [{"to": [{"email": to}]}],
                    "from": {"email": from_email, "name": "FoundryOps"},
                    "subject": subject,
                    "content": [{"type": "text/plain", "value": body}],
                },
                timeout=10.0,
            )

            if response.status_code not in (200, 202):
                log.error(f"SendGrid error: {response.status_code} - {response.text}")
                raise Exception(f"SendGrid failed: {response.status_code}")

    except Exception as e:
        log.error(f"SendGrid send failed: {e}")
        raise


async def _send_via_ses(to: str, subject: str, body: str) -> None:
    """Send email via AWS SES"""
    try:
        import boto3

        from_email = os.getenv("SES_FROM_EMAIL", "notifications@foundrymatch.com")
        region = os.getenv("AWS_REGION", "us-east-1")

        ses_client = boto3.client("ses", region_name=region)

        response = ses_client.send_email(
            Source=from_email,
            Destination={"ToAddresses": [to]},
            Message={"Subject": {"Data": subject}, "Body": {"Text": {"Data": body}}},
        )

        log.info(f"SES email sent: {response['MessageId']}")

    except Exception as e:
        log.error(f"SES send failed: {e}")
        raise


async def _send_via_smtp(to: str, subject: str, body: str) -> None:
    """Send email via SMTP"""
    try:
        import smtplib
        from email.mime.text import MIMEText
        from email.mime.multipart import MIMEMultipart

        smtp_host = os.getenv("SMTP_HOST", "smtp.gmail.com")
        smtp_port = int(os.getenv("SMTP_PORT", "587"))
        smtp_user = os.getenv("SMTP_USER")
        smtp_password = os.getenv("SMTP_PASSWORD")
        from_email = os.getenv("SMTP_FROM_EMAIL", smtp_user)

        if not smtp_user or not smtp_password:
            log.warning("SMTP credentials not configured")
            return

        msg = MIMEMultipart()
        msg["From"] = from_email
        msg["To"] = to
        msg["Subject"] = subject
        msg.attach(MIMEText(body, "plain"))

        with smtplib.SMTP(smtp_host, smtp_port) as server:
            server.starttls()
            server.login(smtp_user, smtp_password)
            server.send_message(msg)

        log.info(f"SMTP email sent to {to}")

    except Exception as e:
        log.error(f"SMTP send failed: {e}")
        raise
